@@ -19,7 +19,9 @@ fn cast_impl(name: &str, chunks: &[ArrayRef], dtype: &DataType) -> Result<Series
     let out = Series::try_from((name, chunks))?;
     use DataType::*;
     let out = match dtype {
-        Date | Datetime => out.into_date(),
+        Date => out.into_date(),
+        Datetime(tu, tz) => out.into_datetime(*tu, tz.clone()),
+        Duration(tu) => out.into_duration(*tu),
         #[cfg(feature = "dtype-time")]
         Time => out.into_time(),
         _ => out,
@@ -167,7 +169,7 @@ mod test {
     #[cfg(feature = "dtype-categorical")]
     fn test_cast_noop() {
         // check if we can cast categorical twice without panic
-        let ca = Utf8Chunked::new_from_slice("foo", &["bar", "ham"]);
+        let ca = Utf8Chunked::new("foo", &["bar", "ham"]);
         let out = ca.cast(&DataType::Categorical).unwrap();
         let out = out.cast(&DataType::Categorical).unwrap();
         assert_eq!(out.dtype(), &DataType::Categorical)
