@@ -1,14 +1,18 @@
-pub mod builder;
-mod is_valid;
-mod iterator;
-
-pub use crate::prelude::*;
-use arrow::bitmap::Bitmap;
 use std::any::Any;
 use std::fmt::{Debug, Display};
 use std::hash::Hash;
 use std::sync::Arc;
 
+use arrow::bitmap::Bitmap;
+
+pub use crate::prelude::*;
+
+pub mod builder;
+#[cfg(feature = "object")]
+pub(crate) mod extension;
+mod is_valid;
+mod iterator;
+
 #[derive(Debug, Clone)]
 pub struct ObjectArray<T>
 where
@@ -101,12 +105,20 @@ where
     }
 
     fn slice(&self, offset: usize, length: usize) -> Box<dyn Array> {
+        assert!(
+            offset + length <= self.len(),
+            "the offset of the new Buffer cannot exceed the existing length"
+        );
+        unsafe { self.slice_unchecked(offset, length) }
+    }
+
+    unsafe fn slice_unchecked(&self, offset: usize, length: usize) -> Box<dyn Array> {
         let mut new = self.clone();
         let len = std::cmp::min(new.len - offset, length);
 
         new.len = len;
         new.offset = offset;
-        new.null_bitmap = new.null_bitmap.map(|x| x.slice(offset, len));
+        new.null_bitmap = new.null_bitmap.map(|x| x.slice_unchecked(offset, len));
         Box::new(new)
     }
 
@@ -114,8 +126,8 @@ where
         self.len
     }
 
-    fn validity(&self) -> &Option<Bitmap> {
-        &self.null_bitmap
+    fn validity(&self) -> Option<&Bitmap> {
+        self.null_bitmap.as_ref()
     }
     fn with_validity(&self, validity: Option<Bitmap>) -> Box<dyn Array> {
         let mut arr = self.clone();
