@@ -1,13 +1,28 @@
 //! Implementations of the ChunkApply Trait.
 use crate::prelude::*;
 use crate::utils::{CustomIterTools, NoNull};
-use arrow::array::{Array, ArrayRef, BooleanArray, LargeStringArray, PrimitiveArray};
+use arrow::array::{ArrayRef, BooleanArray, PrimitiveArray};
+use polars_arrow::array::PolarsArray;
+use polars_arrow::trusted_len::PushUnchecked;
 use std::borrow::Cow;
 use std::convert::TryFrom;
 
+macro_rules! try_apply {
+    ($self:expr, $f:expr) => {{
+        if !$self.has_validity() {
+            $self.into_no_null_iter().map($f).collect()
+        } else {
+            $self
+                .into_iter()
+                .map(|opt_v| opt_v.map($f).transpose())
+                .collect()
+        }
+    }};
+}
+
 macro_rules! apply {
     ($self:expr, $f:expr) => {{
-        if $self.null_count() == 0 {
+        if !$self.has_validity() {
             $self.into_no_null_iter().map($f).collect_trusted()
         } else {
             $self
@@ -20,7 +35,7 @@ macro_rules! apply {
 
 macro_rules! apply_enumerate {
     ($self:expr, $f:expr) => {{
-        if $self.null_count() == 0 {
+        if !$self.has_validity() {
             $self
                 .into_no_null_iter()
                 .enumerate()
@@ -45,41 +60,33 @@ where
         F: Fn(T::Native) -> S::Native + Copy,
         S: PolarsNumericType,
     {
-        let mut ca: ChunkedArray<S> = self
+        let chunks = self
             .data_views()
-            .zip(self.null_bits())
-            .map(|(slice, (_null_count, opt_buffer))| {
-                let vec: AlignedVec<_> = slice.iter().copied().map(f).collect_trusted();
-                (vec, opt_buffer.cloned())
+            .zip(self.iter_validities())
+            .map(|(slice, validity)| {
+                let values = Vec::<_>::from_trusted_len_iter(slice.iter().map(|&v| f(v)));
+                to_array::<S>(values, validity.cloned())
             })
             .collect();
-        ca.rename(self.name());
-        ca
+        ChunkedArray::<S>::new_from_chunks(self.name(), chunks)
     }
 
     fn branch_apply_cast_numeric_no_null<F, S>(&self, f: F) -> ChunkedArray<S>
     where
-        F: Fn(Option<T::Native>) -> S::Native + Copy,
+        F: Fn(Option<T::Native>) -> S::Native,
         S: PolarsNumericType,
     {
         let chunks = self
             .downcast_iter()
             .map(|array| {
-                let av: AlignedVec<_> = if array.null_count() == 0 {
-                    array
-                        .values()
-                        .iter()
-                        .map(|&v| f(Some(v)))
-                        .trust_my_length(array.len())
-                        .collect_trusted()
+                let values = if !array.has_validity() {
+                    let values = array.values().iter().map(|&v| f(Some(v)));
+                    Vec::<_>::from_trusted_len_iter(values)
                 } else {
-                    array
-                        .into_iter()
-                        .map(f)
-                        .trust_my_length(array.len())
-                        .collect_trusted()
+                    let values = array.into_iter().map(|v| f(v.copied()));
+                    Vec::<_>::from_trusted_len_iter(values)
                 };
-                Arc::new(av.into_primitive_array::<S>(None)) as ArrayRef
+                to_array::<S>(values, None)
             })
             .collect();
         ChunkedArray::<S>::new_from_chunks(self.name(), chunks)
@@ -88,18 +95,35 @@ where
     fn apply<F>(&'a self, f: F) -> Self
     where
         F: Fn(T::Native) -> T::Native + Copy,
+    {
+        let chunks = self
+            .data_views()
+            .into_iter()
+            .zip(self.iter_validities())
+            .map(|(slice, validity)| {
+                let values = slice.iter().copied().map(f);
+                let values = Vec::<_>::from_trusted_len_iter(values);
+                to_array::<T>(values, validity.cloned())
+            })
+            .collect();
+        ChunkedArray::<T>::new_from_chunks(self.name(), chunks)
+    }
+
+    fn try_apply<F>(&'a self, f: F) -> Result<Self>
+    where
+        F: Fn(T::Native) -> Result<T::Native> + Copy,
     {
         let mut ca: ChunkedArray<T> = self
             .data_views()
             .into_iter()
-            .zip(self.null_bits())
-            .map(|(slice, (_null_count, opt_buffer))| {
-                let vec: AlignedVec<_> = slice.iter().copied().map(f).collect_trusted();
-                (vec, opt_buffer.cloned())
+            .zip(self.iter_validities())
+            .map(|(slice, validity)| {
+                let vec: Result<Vec<_>> = slice.iter().copied().map(f).collect();
+                Ok((vec?, validity.cloned()))
             })
-            .collect();
+            .collect::<Result<_>>()?;
         ca.rename(self.name());
-        ca
+        Ok(ca)
     }
 
     fn apply_on_opt<F>(&'a self, f: F) -> Self
@@ -109,10 +133,9 @@ where
         let chunks = self
             .downcast_iter()
             .map(|arr| {
-                let iter = arr.into_iter().map(f);
-                // Safety:
-                // array's length is trusted
-                let arr = unsafe { PrimitiveArray::<T>::from_trusted_len_iter(iter) };
+                let iter = arr.into_iter().map(|opt_v| f(opt_v.copied()));
+                let arr = PrimitiveArray::<T::Native>::from_trusted_len_iter(iter)
+                    .to(T::get_dtype().to_arrow());
                 Arc::new(arr) as ArrayRef
             })
             .collect();
@@ -123,7 +146,7 @@ where
     where
         F: Fn((usize, T::Native)) -> T::Native + Copy,
     {
-        if self.null_count() == 0 {
+        if !self.has_validity() {
             let ca: NoNull<_> = self
                 .into_no_null_iter()
                 .enumerate()
@@ -131,12 +154,15 @@ where
                 .collect_trusted();
             ca.into_inner()
         } else {
-            self.downcast_iter()
-                .flatten()
-                .trust_my_length(self.len())
-                .enumerate()
-                .map(|(idx, opt_v)| opt_v.map(|v| f((idx, v))))
-                .collect_trusted()
+            // we know that we only iterate over length == self.len()
+            unsafe {
+                self.downcast_iter()
+                    .flatten()
+                    .trust_my_length(self.len())
+                    .enumerate()
+                    .map(|(idx, opt_v)| opt_v.map(|v| f((idx, *v))))
+                    .collect_trusted()
+            }
         }
     }
 
@@ -144,12 +170,15 @@ where
     where
         F: Fn((usize, Option<T::Native>)) -> Option<T::Native> + Copy,
     {
-        self.downcast_iter()
-            .flatten()
-            .trust_my_length(self.len())
-            .enumerate()
-            .map(f)
-            .collect_trusted()
+        // we know that we only iterate over length == self.len()
+        unsafe {
+            self.downcast_iter()
+                .flatten()
+                .trust_my_length(self.len())
+                .enumerate()
+                .map(|(idx, v)| f((idx, v.copied())))
+                .collect_trusted()
+        }
     }
     fn apply_to_slice<F, V>(&'a self, f: F, slice: &mut [V])
     where
@@ -163,7 +192,7 @@ where
                 // Safety:
                 // length asserted above
                 let item = unsafe { slice.get_unchecked_mut(idx) };
-                *item = f(opt_val, item);
+                *item = f(opt_val.copied(), item);
                 idx += 1;
             })
         });
@@ -177,11 +206,10 @@ impl<'a> ChunkApply<'a, bool, bool> for BooleanChunked {
         S: PolarsNumericType,
     {
         self.apply_kernel_cast(|array| {
-            let av: AlignedVec<_> = (0..array.len())
-                .map(|idx| unsafe { f(array.value_unchecked(idx)) })
-                .collect_trusted();
-            let null_bit_buffer = array.data_ref().null_buffer().cloned();
-            Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
+            let values = array.values().iter().map(f);
+            let values = Vec::<_>::from_trusted_len_iter(values);
+            let validity = array.validity().cloned();
+            to_array::<S>(values, validity)
         })
     }
 
@@ -191,12 +219,8 @@ impl<'a> ChunkApply<'a, bool, bool> for BooleanChunked {
         S: PolarsNumericType,
     {
         self.apply_kernel_cast(|array| {
-            let av: AlignedVec<_> = array
-                .into_iter()
-                .map(f)
-                .trust_my_length(array.len())
-                .collect_trusted();
-            Arc::new(av.into_primitive_array::<S>(None)) as ArrayRef
+            let values = Vec::<_>::from_trusted_len_iter(array.into_iter().map(f));
+            to_array::<S>(values, None)
         })
     }
 
@@ -207,6 +231,13 @@ impl<'a> ChunkApply<'a, bool, bool> for BooleanChunked {
         apply!(self, f)
     }
 
+    fn try_apply<F>(&self, f: F) -> Result<Self>
+    where
+        F: Fn(bool) -> Result<bool> + Copy,
+    {
+        try_apply!(self, f)
+    }
+
     fn apply_on_opt<F>(&'a self, f: F) -> Self
     where
         F: Fn(Option<bool>) -> Option<bool> + Copy,
@@ -257,11 +288,9 @@ impl<'a> ChunkApply<'a, &'a str, Cow<'a, str>> for Utf8Chunked {
             .downcast_iter()
             .into_iter()
             .map(|array| {
-                let av: AlignedVec<_> = (0..array.len())
-                    .map(|idx| unsafe { f(array.value_unchecked(idx)) })
-                    .collect_trusted();
-                let null_bit_buffer = array.data_ref().null_buffer().cloned();
-                Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
+                let values = array.values_iter().map(f);
+                let values = Vec::<_>::from_trusted_len_iter(values);
+                to_array::<S>(values, array.validity().cloned())
             })
             .collect();
         ChunkedArray::new_from_chunks(self.name(), chunks)
@@ -276,13 +305,9 @@ impl<'a> ChunkApply<'a, &'a str, Cow<'a, str>> for Utf8Chunked {
             .downcast_iter()
             .into_iter()
             .map(|array| {
-                let av: AlignedVec<_> = array
-                    .into_iter()
-                    .map(f)
-                    .trust_my_length(array.len())
-                    .collect_trusted();
-                let null_bit_buffer = array.data_ref().null_buffer().cloned();
-                Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
+                let values = array.into_iter().map(f);
+                let values = Vec::<_>::from_trusted_len_iter(values);
+                to_array::<S>(values, array.validity().cloned())
             })
             .collect();
         ChunkedArray::new_from_chunks(self.name(), chunks)
@@ -295,6 +320,13 @@ impl<'a> ChunkApply<'a, &'a str, Cow<'a, str>> for Utf8Chunked {
         apply!(self, f)
     }
 
+    fn try_apply<F>(&'a self, f: F) -> Result<Self>
+    where
+        F: Fn(&'a str) -> Result<Cow<'a, str>> + Copy,
+    {
+        try_apply!(self, f)
+    }
+
     fn apply_on_opt<F>(&'a self, f: F) -> Self
     where
         F: Fn(Option<&'a str>) -> Option<Cow<'a, str>> + Copy,
@@ -344,11 +376,7 @@ impl ChunkApplyKernel<BooleanArray> for BooleanChunked {
     where
         F: Fn(&BooleanArray) -> ArrayRef,
     {
-        let chunks = self
-            .downcast_iter()
-            .into_iter()
-            .map(|array| f(array))
-            .collect();
+        let chunks = self.downcast_iter().into_iter().map(f).collect();
         Self::new_from_chunks(self.name(), chunks)
     }
 
@@ -357,28 +385,24 @@ impl ChunkApplyKernel<BooleanArray> for BooleanChunked {
         F: Fn(&BooleanArray) -> ArrayRef,
         S: PolarsDataType,
     {
-        let chunks = self
-            .downcast_iter()
-            .into_iter()
-            .map(|array| f(array))
-            .collect();
+        let chunks = self.downcast_iter().into_iter().map(f).collect();
         ChunkedArray::<S>::new_from_chunks(self.name(), chunks)
     }
 }
 
-impl<T> ChunkApplyKernel<PrimitiveArray<T>> for ChunkedArray<T>
+impl<T> ChunkApplyKernel<PrimitiveArray<T::Native>> for ChunkedArray<T>
 where
     T: PolarsNumericType,
 {
     fn apply_kernel<F>(&self, f: F) -> Self
     where
-        F: Fn(&PrimitiveArray<T>) -> ArrayRef,
+        F: Fn(&PrimitiveArray<T::Native>) -> ArrayRef,
     {
         self.apply_kernel_cast(f)
     }
     fn apply_kernel_cast<F, S>(&self, f: F) -> ChunkedArray<S>
     where
-        F: Fn(&PrimitiveArray<T>) -> ArrayRef,
+        F: Fn(&PrimitiveArray<T::Native>) -> ArrayRef,
         S: PolarsDataType,
     {
         let chunks = self.downcast_iter().into_iter().map(f).collect();
@@ -414,15 +438,14 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
             .downcast_iter()
             .into_iter()
             .map(|array| {
-                let av: AlignedVec<_> = (0..array.len())
+                let values: Vec<_> = (0..array.len())
                     .map(|idx| {
-                        let arrayref = unsafe { array.value_unchecked(idx) };
+                        let arrayref: ArrayRef = unsafe { array.value_unchecked(idx) }.into();
                         let series = Series::try_from(("", arrayref)).unwrap();
                         f(series)
                     })
                     .collect_trusted();
-                let null_bit_buffer = array.data_ref().null_buffer().cloned();
-                Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
+                to_array::<S>(values, array.validity().cloned())
             })
             .collect();
         ChunkedArray::new_from_chunks(self.name(), chunks)
@@ -437,21 +460,20 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
             .downcast_iter()
             .into_iter()
             .map(|array| {
-                let av: AlignedVec<_> = (0..array.len())
-                    .map(|idx| {
-                        let v = if array.is_valid(idx) {
-                            let arrayref = unsafe { array.value_unchecked(idx) };
-                            let series = Series::try_from(("", arrayref)).unwrap();
-                            Some(series)
-                        } else {
-                            None
-                        };
+                let values = array.iter().map(|x| {
+                    let x = x.map(|x| {
+                        let x: ArrayRef = x.into();
+                        Series::try_from(("", x)).unwrap()
+                    });
+                    f(x)
+                });
+                let len = values.len();
 
-                        f(v)
-                    })
-                    .collect_trusted();
-                let null_bit_buffer = array.data_ref().null_buffer().cloned();
-                Arc::new(av.into_primitive_array::<S>(null_bit_buffer)) as ArrayRef
+                // we know the iterators len
+                unsafe {
+                    let values = Vec::<_>::from_trusted_len_iter(values.trust_my_length(len));
+                    to_array::<S>(values, array.validity().cloned())
+                }
             })
             .collect();
         ChunkedArray::new_from_chunks(self.name(), chunks)
@@ -462,13 +484,57 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
     where
         F: Fn(Series) -> Series + Copy,
     {
-        apply!(self, f)
+        if self.is_empty() {
+            return self.clone();
+        }
+        let mut fast_explode = true;
+        let mut function = |s: Series| {
+            let out = f(s);
+            if out.is_empty() {
+                fast_explode = false;
+            }
+            out
+        };
+        let mut ca: ListChunked = apply!(self, &mut function);
+        if fast_explode {
+            ca.set_fast_explode()
+        }
+        ca
+    }
+
+    fn try_apply<F>(&'a self, f: F) -> Result<Self>
+    where
+        F: Fn(Series) -> Result<Series> + Copy,
+    {
+        if self.is_empty() {
+            return Ok(self.clone());
+        }
+
+        let mut fast_explode = true;
+        let mut function = |s: Series| {
+            let out = f(s);
+            if let Ok(out) = &out {
+                if out.is_empty() {
+                    fast_explode = false;
+                }
+            }
+            out
+        };
+        let ca: Result<ListChunked> = try_apply!(self, &mut function);
+        let mut ca = ca?;
+        if fast_explode {
+            ca.set_fast_explode()
+        }
+        Ok(ca)
     }
 
     fn apply_on_opt<F>(&'a self, f: F) -> Self
     where
         F: Fn(Option<Series>) -> Option<Series> + Copy,
     {
+        if self.is_empty() {
+            return self.clone();
+        }
         self.into_iter().map(f).collect_trusted()
     }
 
@@ -477,7 +543,22 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
     where
         F: Fn((usize, Series)) -> Series + Copy,
     {
-        apply_enumerate!(self, f)
+        if self.is_empty() {
+            return self.clone();
+        }
+        let mut fast_explode = true;
+        let mut function = |(idx, s)| {
+            let out = f((idx, s));
+            if out.is_empty() {
+                fast_explode = false;
+            }
+            out
+        };
+        let mut ca: ListChunked = apply_enumerate!(self, function);
+        if fast_explode {
+            ca.set_fast_explode()
+        }
+        ca
     }
 
     /// Apply a closure elementwise. The closure gets the index of the element as first argument.
@@ -485,7 +566,24 @@ impl<'a> ChunkApply<'a, Series, Series> for ListChunked {
     where
         F: Fn((usize, Option<Series>)) -> Option<Series> + Copy,
     {
-        self.into_iter().enumerate().map(f).collect_trusted()
+        if self.is_empty() {
+            return self.clone();
+        }
+        let mut fast_explode = true;
+        let function = |(idx, s)| {
+            let out = f((idx, s));
+            if let Some(out) = &out {
+                if out.is_empty() {
+                    fast_explode = false;
+                }
+            }
+            out
+        };
+        let mut ca: ListChunked = self.into_iter().enumerate().map(function).collect_trusted();
+        if fast_explode {
+            ca.set_fast_explode()
+        }
+        ca
     }
 
     fn apply_to_slice<F, T>(&'a self, f: F, slice: &mut [T])
@@ -539,6 +637,13 @@ where
         ca
     }
 
+    fn try_apply<F>(&'a self, _f: F) -> Result<Self>
+    where
+        F: Fn(&'a T) -> Result<T> + Copy,
+    {
+        todo!()
+    }
+
     fn apply_on_opt<F>(&'a self, f: F) -> Self
     where
         F: Fn(Option<&'a T>) -> Option<T> + Copy,
