@@ -9,7 +9,6 @@ use crate::datatypes::PlHashSet;
 use crate::frame::groupby::{GroupTuples, IntoGroupTuples};
 use crate::prelude::*;
 use crate::utils::NoNull;
-use itertools::Itertools;
 use rayon::prelude::*;
 use std::hash::Hash;
 #[cfg(feature = "dtype-categorical")]
@@ -111,12 +110,12 @@ where
     a.collect()
 }
 
-fn arg_unique<T>(a: impl Iterator<Item = T>, capacity: usize) -> AlignedVec<u32>
+fn arg_unique<T>(a: impl Iterator<Item = T>, capacity: usize) -> Vec<u32>
 where
     T: Hash + Eq,
 {
     let mut set = PlHashSet::new();
-    let mut unique = AlignedVec::with_capacity(capacity);
+    let mut unique = Vec::with_capacity(capacity);
     a.enumerate().for_each(|(idx, val)| {
         if set.insert(val) {
             unique.push(idx as u32)
@@ -179,7 +178,7 @@ macro_rules! impl_value_counts {
         counts.rename("counts");
         let cols = vec![values.into_series(), counts.into_inner().into_series()];
         let df = DataFrame::new_no_checks(cols);
-        df.sort("counts", true)
+        df.sort(&["counts"], true)
     }};
 }
 
@@ -214,7 +213,11 @@ where
     }
 
     fn n_unique(&self) -> Result<usize> {
-        Ok(fill_set(self.into_iter()).len())
+        if self.null_count() > 0 {
+            Ok(fill_set(self.into_iter().flatten()).len() + 1)
+        } else {
+            Ok(fill_set(self.into_no_null_iter()).len())
+        }
     }
 
     #[cfg(feature = "mode")]
@@ -251,7 +254,11 @@ impl ChunkUnique<Utf8Type> for Utf8Chunked {
     }
 
     fn n_unique(&self) -> Result<usize> {
-        Ok(fill_set(self.into_iter()).len())
+        if self.null_count() > 0 {
+            Ok(fill_set(self.into_iter().flatten()).len() + 1)
+        } else {
+            Ok(fill_set(self.into_no_null_iter()).len())
+        }
     }
 
     #[cfg(feature = "mode")]
@@ -264,11 +271,17 @@ impl ChunkUnique<Utf8Type> for Utf8Chunked {
 impl ChunkUnique<CategoricalType> for CategoricalChunked {
     fn unique(&self) -> Result<Self> {
         let cat_map = self.categorical_map.as_ref().unwrap();
-        let mut ca = match &**cat_map {
-            RevMapping::Local(a) => UInt32Chunked::new_from_iter(self.name(), 0..(a.len() as u32)),
-            RevMapping::Global(map, _, _) => {
-                UInt32Chunked::new_from_iter(self.name(), map.keys().copied())
+        let mut ca = if self.can_fast_unique() {
+            match &**cat_map {
+                RevMapping::Local(a) => {
+                    UInt32Chunked::new_from_iter(self.name(), 0..(a.len() as u32))
+                }
+                RevMapping::Global(map, _, _) => {
+                    UInt32Chunked::new_from_iter(self.name(), map.keys().copied())
+                }
             }
+        } else {
+            self.deref().unique()?
         };
         ca.categorical_map = self.categorical_map.clone();
         Ok(ca.into())
@@ -289,7 +302,11 @@ impl ChunkUnique<CategoricalType> for CategoricalChunked {
         impl_value_counts!(self)
     }
     fn n_unique(&self) -> Result<usize> {
-        Ok(self.categorical_map.as_ref().unwrap().len())
+        if self.can_fast_unique() {
+            Ok(self.categorical_map.as_ref().unwrap().len())
+        } else {
+            self.deref().n_unique()
+        }
     }
     #[cfg(feature = "mode")]
     fn mode(&self) -> Result<Self> {
@@ -301,7 +318,7 @@ impl ChunkUnique<CategoricalType> for CategoricalChunked {
 fn dummies_helper(mut groups: Vec<u32>, len: usize, name: &str) -> UInt8Chunked {
     groups.sort_unstable();
 
-    let mut av: AlignedVec<_> = (0..len).map(|_| 0u8).collect();
+    let mut av: Vec<_> = (0..len).map(|_| 0u8).collect();
 
     for idx in groups {
         let elem = unsafe { av.get_unchecked_mut(idx as usize) };
@@ -316,7 +333,7 @@ fn dummies_helper(mut groups: Vec<u32>, len: usize, name: &str) -> Int32Chunked
     groups.sort_unstable();
 
     // let mut group_member_iter = groups.into_iter();
-    let mut av: AlignedVec<_> = (0..len).map(|_| 0i32).collect();
+    let mut av: Vec<_> = (0..len).map(|_| 0i32).collect();
 
     for idx in groups {
         let elem = unsafe { av.get_unchecked_mut(idx as usize) };
@@ -326,11 +343,9 @@ fn dummies_helper(mut groups: Vec<u32>, len: usize, name: &str) -> Int32Chunked
     ChunkedArray::new_from_aligned_vec(name, av)
 }
 
-fn sort_columns(columns: Vec<Series>) -> Vec<Series> {
+fn sort_columns(mut columns: Vec<Series>) -> Vec<Series> {
+    columns.sort_by(|a, b| a.name().partial_cmp(b.name()).unwrap());
     columns
-        .into_iter()
-        .sorted_by_key(|s| s.name().to_string())
-        .collect()
 }
 
 impl ToDummies<Utf8Type> for Utf8Chunked {
@@ -397,7 +412,7 @@ impl ChunkUnique<BooleanType> for BooleanChunked {
                 unique.push(v)
             }
         }
-        Ok(ChunkedArray::new_from_opt_slice(self.name(), &unique))
+        Ok(ChunkedArray::new(self.name(), &unique))
     }
 
     fn arg_unique(&self) -> Result<UInt32Chunked> {
@@ -545,23 +560,25 @@ mod is_first {
 #[cfg(test)]
 mod test {
     use crate::prelude::*;
-    use itertools::Itertools;
 
     #[test]
     fn unique() {
         let ca = ChunkedArray::<Int32Type>::new_from_slice("a", &[1, 2, 3, 2, 1]);
         assert_eq!(
-            ca.unique().unwrap().sort(false).into_iter().collect_vec(),
+            ca.unique()
+                .unwrap()
+                .sort(false)
+                .into_iter()
+                .collect::<Vec<_>>(),
             vec![Some(1), Some(2), Some(3)]
         );
         let ca = BooleanChunked::new_from_slice("a", &[true, false, true]);
         assert_eq!(
-            ca.unique().unwrap().into_iter().collect_vec(),
+            ca.unique().unwrap().into_iter().collect::<Vec<_>>(),
             vec![Some(true), Some(false)]
         );
 
-        let ca =
-            Utf8Chunked::new_from_opt_slice("", &[Some("a"), None, Some("a"), Some("b"), None]);
+        let ca = Utf8Chunked::new("", &[Some("a"), None, Some("a"), Some("b"), None]);
         assert_eq!(
             Vec::from(&ca.unique().unwrap().sort(false)),
             &[None, Some("a"), Some("b")]
@@ -572,7 +589,7 @@ mod test {
     fn arg_unique() {
         let ca = ChunkedArray::<Int32Type>::new_from_slice("a", &[1, 2, 1, 1, 3]);
         assert_eq!(
-            ca.arg_unique().unwrap().into_iter().collect_vec(),
+            ca.arg_unique().unwrap().into_iter().collect::<Vec<_>>(),
             vec![Some(0), Some(1), Some(4)]
         );
     }
@@ -595,7 +612,7 @@ mod test {
     #[test]
     #[cfg(feature = "is_first")]
     fn is_first() {
-        let ca = UInt32Chunked::new_from_opt_slice(
+        let ca = UInt32Chunked::new(
             "a",
             &[Some(1), Some(2), Some(1), Some(1), None, Some(3), None],
         );
