@@ -11,11 +11,14 @@ use crate::chunked_array::categorical::RevMapping;
 pub use crate::chunked_array::logical::*;
 #[cfg(feature = "object")]
 use crate::chunked_array::object::PolarsObjectSafe;
+use crate::chunked_array::ops::sort::PlIsNan;
 use crate::prelude::*;
 use ahash::RandomState;
+use arrow::compute::arithmetics::basic::NativeArithmetics;
 use arrow::compute::comparison::Simd8;
 use arrow::datatypes::IntegerType;
-pub use arrow::datatypes::{DataType as ArrowDataType, TimeUnit};
+pub use arrow::datatypes::{DataType as ArrowDataType, TimeUnit as ArrowTimeUnit};
+use arrow::error::ArrowError;
 use arrow::types::simd::Simd;
 use arrow::types::NativeType;
 use num::{Bounded, FromPrimitive, Num, NumCast, Zero};
@@ -61,7 +64,8 @@ impl_polars_datatype!(Int64Type, Int64, i64);
 impl_polars_datatype!(Float32Type, Float32, f32);
 impl_polars_datatype!(Float64Type, Float64, f64);
 impl_polars_datatype!(DateType, Date, i32);
-impl_polars_datatype!(DatetimeType, Datetime, i64);
+impl_polars_datatype!(DatetimeType, Unknown, i64);
+impl_polars_datatype!(DurationType, Unknown, i64);
 impl_polars_datatype!(TimeType, Time, i64);
 
 impl PolarsDataType for Utf8Type {
@@ -144,6 +148,8 @@ pub trait NumericNative:
     + AddAssign
     + Bounded
     + FromPrimitive
+    + PlIsNan
+    + NativeArithmetics
 {
 }
 impl NumericNative for i8 {}
@@ -237,9 +243,12 @@ pub enum AnyValue<'a> {
     #[cfg(feature = "dtype-date")]
     Date(i32),
     /// A 64-bit date representing the elapsed time since UNIX epoch (1970-01-01)
-    /// in milliseconds (64 bits).
+    /// in nanoseconds (64 bits).
     #[cfg(feature = "dtype-datetime")]
-    Datetime(i64),
+    Datetime(i64, TimeUnit, &'a Option<TimeZone>),
+    // A 64-bit integer representing difference between date-times in nanoseconds or milliseconds
+    #[cfg(feature = "dtype-duration")]
+    Duration(i64, TimeUnit),
     /// A 64-bit time representing the elapsed time since midnight in nanoseconds
     #[cfg(feature = "dtype-time")]
     Time(i64),
@@ -322,8 +331,23 @@ impl<'a> AnyValue<'a> {
         match self {
             #[cfg(feature = "dtype-date")]
             AnyValue::Int32(v) => AnyValue::Date(v),
+            AnyValue::Null => AnyValue::Null,
+            dt => panic!("cannot create date from other type. dtype: {}", dt),
+        }
+    }
+    pub(crate) fn into_datetime(self, tu: TimeUnit, tz: &'a Option<TimeZone>) -> Self {
+        match self {
             #[cfg(feature = "dtype-datetime")]
-            AnyValue::Int64(v) => AnyValue::Datetime(v),
+            AnyValue::Int64(v) => AnyValue::Datetime(v, tu, tz),
+            AnyValue::Null => AnyValue::Null,
+            dt => panic!("cannot create date from other type. dtype: {}", dt),
+        }
+    }
+
+    pub(crate) fn into_duration(self, tu: TimeUnit) -> Self {
+        match self {
+            #[cfg(feature = "dtype-duration")]
+            AnyValue::Int64(v) => AnyValue::Duration(v, tu),
             AnyValue::Null => AnyValue::Null,
             dt => panic!("cannot create date from other type. dtype: {}", dt),
         }
@@ -338,6 +362,7 @@ impl<'a> AnyValue<'a> {
         }
     }
 
+    #[must_use]
     pub fn add<'b>(&self, rhs: &AnyValue<'b>) -> Self {
         use AnyValue::*;
         match (self, rhs) {
@@ -352,6 +377,37 @@ impl<'a> AnyValue<'a> {
             _ => todo!(),
         }
     }
+
+    /// Try to coerce to an AnyValue with static lifetime.
+    /// This can be done if it does not borrow any values.
+    pub fn to_static(&self) -> Result<AnyValue<'static>> {
+        use AnyValue::*;
+        let av = match self {
+            Null => AnyValue::Null,
+            Int8(v) => AnyValue::Int8(*v),
+            Int16(v) => AnyValue::Int16(*v),
+            Int32(v) => AnyValue::Int32(*v),
+            Int64(v) => AnyValue::Int64(*v),
+            UInt8(v) => AnyValue::UInt8(*v),
+            UInt16(v) => AnyValue::UInt16(*v),
+            UInt32(v) => AnyValue::UInt32(*v),
+            UInt64(v) => AnyValue::UInt64(*v),
+            Boolean(v) => AnyValue::Boolean(*v),
+            Float32(v) => AnyValue::Float32(*v),
+            Float64(v) => AnyValue::Float64(*v),
+            #[cfg(feature = "dtype-date")]
+            Date(v) => AnyValue::Date(*v),
+            #[cfg(feature = "dtype-time")]
+            Time(v) => AnyValue::Time(*v),
+            List(v) => AnyValue::List(v.clone()),
+            dt => {
+                return Err(PolarsError::ComputeError(
+                    format!("cannot get static AnyValue from {}", dt).into(),
+                ))
+            }
+        };
+        Ok(av)
+    }
 }
 
 impl<'a> From<AnyValue<'a>> for Option<i64> {
@@ -384,12 +440,20 @@ impl Display for DataType {
             DataType::Float64 => "f64",
             DataType::Utf8 => "str",
             DataType::Date => "date",
-            DataType::Datetime => "datetime",
+            DataType::Datetime(tu, tz) => {
+                let s = match tz {
+                    None => format!("datetime[{}]", tu),
+                    Some(tz) => format!("datetime[{}, {}]", tu, tz),
+                };
+                return f.write_str(&s);
+            }
+            DataType::Duration(tu) => return write!(f, "duration[{}]", tu),
             DataType::Time => "time",
             DataType::List(tp) => return write!(f, "list [{}]", tp),
             #[cfg(feature = "object")]
             DataType::Object(s) => s,
             DataType::Categorical => "cat",
+            DataType::Unknown => unreachable!(),
         };
         f.write_str(s)
     }
@@ -416,7 +480,7 @@ impl PartialEq for AnyValue<'_> {
             #[cfg(all(feature = "dtype-datetime", feature = "dtype-date"))]
             (Date(l), Date(r)) => l == r,
             #[cfg(all(feature = "dtype-datetime", feature = "dtype-date"))]
-            (Datetime(l), Datetime(r)) => l == r,
+            (Datetime(l, tul, tzl), Datetime(r, tur, tzr)) => l == r && tul == tur && tzl == tzr,
             (Boolean(l), Boolean(r)) => l == r,
             (List(_), List(_)) => panic!("eq between list series not supported"),
             #[cfg(feature = "object")]
@@ -458,6 +522,57 @@ impl PartialOrd for AnyValue<'_> {
     }
 }
 
+#[derive(Copy, Clone, Debug, PartialEq, Eq, Hash)]
+pub enum TimeUnit {
+    Nanoseconds,
+    Milliseconds,
+}
+
+impl From<&ArrowTimeUnit> for TimeUnit {
+    fn from(tu: &ArrowTimeUnit) -> Self {
+        match tu {
+            ArrowTimeUnit::Millisecond => TimeUnit::Milliseconds,
+            ArrowTimeUnit::Nanosecond => TimeUnit::Nanoseconds,
+            // will be cast
+            ArrowTimeUnit::Microsecond => TimeUnit::Nanoseconds,
+            // will be cast
+            ArrowTimeUnit::Second => TimeUnit::Milliseconds,
+        }
+    }
+}
+
+impl Display for TimeUnit {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            TimeUnit::Nanoseconds => {
+                write!(f, "ns")
+            }
+            TimeUnit::Milliseconds => {
+                write!(f, "ms")
+            }
+        }
+    }
+}
+
+impl TimeUnit {
+    pub fn to_arrow(self) -> ArrowTimeUnit {
+        match self {
+            TimeUnit::Nanoseconds => ArrowTimeUnit::Nanosecond,
+            TimeUnit::Milliseconds => ArrowTimeUnit::Millisecond,
+        }
+    }
+
+    #[cfg(any(feature = "temporal", feature = "dynamic_groupby"))]
+    pub(crate) fn to_polars_time(self) -> polars_time::groupby::TimeUnit {
+        match self {
+            TimeUnit::Nanoseconds => polars_time::groupby::TimeUnit::Nanoseconds,
+            TimeUnit::Milliseconds => polars_time::groupby::TimeUnit::Milliseconds,
+        }
+    }
+}
+
+pub type TimeZone = String;
+
 #[derive(Clone, PartialEq, Eq, Debug, Hash)]
 pub enum DataType {
     Boolean,
@@ -478,7 +593,9 @@ pub enum DataType {
     Date,
     /// A 64-bit date representing the elapsed time since UNIX epoch (1970-01-01)
     /// in milliseconds (64 bits).
-    Datetime,
+    Datetime(TimeUnit, Option<TimeZone>),
+    // 64-bit integer representing difference between times in milliseconds or nanoseconds
+    Duration(TimeUnit),
     /// A 64-bit time representing the elapsed time since midnight in nanoseconds
     Time,
     List(Box<DataType>),
@@ -488,6 +605,8 @@ pub enum DataType {
     Object(&'static str),
     Null,
     Categorical,
+    // some logical types we cannot know statically, e.g. Datetime
+    Unknown,
 }
 
 impl DataType {
@@ -500,11 +619,13 @@ impl DataType {
     }
 
     /// Convert to the physical data type
+    #[must_use]
     pub fn to_physical(&self) -> DataType {
         use DataType::*;
         match self {
             Date => Int32,
-            Datetime => Int64,
+            Datetime(_, _) => Int64,
+            Duration(_) => Int64,
             Time => Int64,
             Categorical => UInt32,
             _ => self.clone(),
@@ -528,8 +649,9 @@ impl DataType {
             Float64 => ArrowDataType::Float64,
             Utf8 => ArrowDataType::LargeUtf8,
             Date => ArrowDataType::Date32,
-            Datetime => ArrowDataType::Timestamp(TimeUnit::Millisecond, None),
-            Time => ArrowDataType::Time64(TimeUnit::Nanosecond),
+            Datetime(unit, tz) => ArrowDataType::Timestamp(unit.to_arrow(), tz.clone()),
+            Duration(unit) => ArrowDataType::Duration(unit.to_arrow()),
+            Time => ArrowDataType::Time64(ArrowTimeUnit::Nanosecond),
             List(dt) => ArrowDataType::LargeList(Box::new(arrow::datatypes::Field::new(
                 "",
                 dt.to_arrow(),
@@ -539,6 +661,7 @@ impl DataType {
             #[cfg(feature = "object")]
             Object(_) => panic!("cannot convert object to arrow"),
             Categorical => ArrowDataType::UInt32,
+            Unknown => unreachable!(),
         }
     }
 }
@@ -550,6 +673,7 @@ impl PartialEq<ArrowDataType> for DataType {
     }
 }
 
+/// Characterizes the name and the [`DataType`] of a column.
 #[derive(Clone, Debug, PartialEq, Hash)]
 pub struct Field {
     name: String,
@@ -557,33 +681,118 @@ pub struct Field {
 }
 
 impl Field {
+    /// Creates a new `Field`.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use polars_core::prelude::*;
+    /// let f1 = Field::new("Fruit name", DataType::Utf8);
+    /// let f2 = Field::new("Lawful", DataType::Boolean);
+    /// let f2 = Field::new("Departure", DataType::Time);
+    /// ```
     pub fn new(name: &str, data_type: DataType) -> Self {
         Field {
             name: name.to_string(),
             data_type,
         }
     }
+
+    /// Returns a reference to the `Field` name.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use polars_core::prelude::*;
+    /// let f = Field::new("Year", DataType::Int32);
+    ///
+    /// assert_eq!(f.name(), "Year");
+    /// ```
     pub fn name(&self) -> &String {
         &self.name
     }
 
+    /// Returns a reference to the `Field` datatype.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use polars_core::prelude::*;
+    /// let f = Field::new("Birthday", DataType::Date);
+    ///
+    /// assert_eq!(f.data_type(), &DataType::Date);
+    /// ```
     pub fn data_type(&self) -> &DataType {
         &self.data_type
     }
 
+    /// Sets the `Field` datatype.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use polars_core::prelude::*;
+    /// let mut f = Field::new("Temperature", DataType::Int32);
+    /// f.coerce(DataType::Float32);
+    ///
+    /// assert_eq!(f, Field::new("Temperature", DataType::Float32));
+    /// ```
     pub fn coerce(&mut self, dtype: DataType) {
         self.data_type = dtype;
     }
 
+    /// Sets the `Field` name.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use polars_core::prelude::*;
+    /// let mut f = Field::new("Atomic number", DataType::UInt32);
+    /// f.set_name("Proton".to_owned());
+    ///
+    /// assert_eq!(f, Field::new("Proton", DataType::UInt32));
+    /// ```
     pub fn set_name(&mut self, name: String) {
         self.name = name;
     }
 
+    /// Converts the `Field` to an `arrow::datatypes::Field`.
+    ///
+    /// # Example
+    ///
+    /// ```rust
+    /// # use polars_core::prelude::*;
+    /// let f = Field::new("Value", DataType::Int64);
+    /// let af = arrow::datatypes::Field::new("Value", arrow::datatypes::DataType::Int64, true);
+    ///
+    /// assert_eq!(f.to_arrow(), af);
+    /// ```
     pub fn to_arrow(&self) -> ArrowField {
         ArrowField::new(&self.name, self.data_type.to_arrow(), true)
     }
 }
 
+#[cfg(feature = "private")]
+pub trait IndexOfSchema {
+    fn index_of(&self, name: &str) -> arrow::error::Result<usize>;
+}
+
+impl IndexOfSchema for ArrowSchema {
+    fn index_of(&self, name: &str) -> arrow::error::Result<usize> {
+        self.fields
+            .iter()
+            .position(|f| f.name == name)
+            .ok_or_else(|| {
+                let valid_fields: Vec<String> =
+                    self.fields.iter().map(|f| f.name.clone()).collect();
+                ArrowError::InvalidArgumentError(format!(
+                    "Unable to get field named \"{}\". Valid fields: {:?}",
+                    name, valid_fields
+                ))
+            })
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Hash, Default)]
 pub struct Schema {
     fields: Vec<Field>,
@@ -645,20 +854,21 @@ impl Schema {
 
     /// Find the index of the column with the given name
     pub fn index_of(&self, name: &str) -> Result<usize> {
-        for i in 0..self.fields.len() {
-            if self.fields[i].name == name {
-                return Ok(i);
-            }
-        }
-        let valid_fields: Vec<String> = self.fields.iter().map(|f| f.name().clone()).collect();
-        Err(PolarsError::NotFound(format!(
-            "Unable to get field named \"{}\". Valid fields: {:?}",
-            name, valid_fields
-        )))
+        self.fields
+            .iter()
+            .position(|f| f.name == name)
+            .ok_or_else(|| {
+                let valid_fields: Vec<String> =
+                    self.fields.iter().map(|f| f.name.clone()).collect();
+                PolarsError::NotFound(format!(
+                    "Unable to get field named \"{}\". Valid fields: {:?}",
+                    name, valid_fields
+                ))
+            })
     }
 
     pub fn to_arrow(&self) -> ArrowSchema {
-        let fields = self
+        let fields: Vec<ArrowField> = self
             .fields
             .iter()
             .map(|f| {
@@ -679,6 +889,7 @@ impl Schema {
                         ArrowDataType::Dictionary(
                             IntegerType::UInt32,
                             Box::new(ArrowDataType::LargeUtf8),
+                            false,
                         ),
                         true,
                     ),
@@ -686,7 +897,7 @@ impl Schema {
                 }
             })
             .collect();
-        ArrowSchema::new(fields)
+        ArrowSchema::from(fields)
     }
 
     pub fn try_merge(schemas: &[Self]) -> Result<Self> {
@@ -740,11 +951,13 @@ impl From<&ArrowDataType> for DataType {
             ArrowDataType::LargeList(f) => DataType::List(Box::new(f.data_type().into())),
             ArrowDataType::List(f) => DataType::List(Box::new(f.data_type().into())),
             ArrowDataType::Date32 => DataType::Date,
-            ArrowDataType::Timestamp(_, _) | ArrowDataType::Date64 => DataType::Datetime,
+            ArrowDataType::Timestamp(tu, tz) => DataType::Datetime(tu.into(), tz.clone()),
+            ArrowDataType::Duration(tu) => DataType::Duration(tu.into()),
+            ArrowDataType::Date64 => DataType::Datetime(TimeUnit::Milliseconds, None),
             ArrowDataType::LargeUtf8 => DataType::Utf8,
             ArrowDataType::Utf8 => DataType::Utf8,
             ArrowDataType::Time64(_) | ArrowDataType::Time32(_) => DataType::Time,
-            ArrowDataType::Dictionary(_, _) => DataType::Categorical,
+            ArrowDataType::Dictionary(_, _, _) => DataType::Categorical,
             ArrowDataType::Extension(name, _, _) if name == "POLARS_EXTENSION_TYPE" => {
                 #[cfg(feature = "object")]
                 {
@@ -762,14 +975,14 @@ impl From<&ArrowDataType> for DataType {
 
 impl From<&ArrowField> for Field {
     fn from(f: &ArrowField) -> Self {
-        Field::new(f.name(), f.data_type().into())
+        Field::new(&f.name, f.data_type().into())
     }
 }
 impl From<&ArrowSchema> for Schema {
     fn from(a_schema: &ArrowSchema) -> Self {
         Schema::new(
             a_schema
-                .fields()
+                .fields
                 .iter()
                 .map(|arrow_f| arrow_f.into())
                 .collect(),
@@ -794,37 +1007,66 @@ mod test {
     #[test]
     fn test_arrow_dtypes_to_polars() {
         let dtypes = [
-            (ArrowDataType::Date64, DataType::Datetime),
             (
-                ArrowDataType::Timestamp(TimeUnit::Nanosecond, None),
-                DataType::Datetime,
+                ArrowDataType::Duration(ArrowTimeUnit::Nanosecond),
+                DataType::Duration(TimeUnit::Nanoseconds),
             ),
             (
-                ArrowDataType::Timestamp(TimeUnit::Microsecond, None),
-                DataType::Datetime,
+                ArrowDataType::Duration(ArrowTimeUnit::Millisecond),
+                DataType::Duration(TimeUnit::Milliseconds),
             ),
             (
-                ArrowDataType::Timestamp(TimeUnit::Millisecond, None),
-                DataType::Datetime,
+                ArrowDataType::Date64,
+                DataType::Datetime(TimeUnit::Milliseconds, None),
             ),
             (
-                ArrowDataType::Timestamp(TimeUnit::Second, None),
-                DataType::Datetime,
+                ArrowDataType::Timestamp(ArrowTimeUnit::Nanosecond, None),
+                DataType::Datetime(TimeUnit::Nanoseconds, None),
             ),
             (
-                ArrowDataType::Timestamp(TimeUnit::Second, Some("".to_string())),
-                DataType::Datetime,
+                ArrowDataType::Timestamp(ArrowTimeUnit::Microsecond, None),
+                DataType::Datetime(TimeUnit::Nanoseconds, None),
+            ),
+            (
+                ArrowDataType::Timestamp(ArrowTimeUnit::Millisecond, None),
+                DataType::Datetime(TimeUnit::Milliseconds, None),
+            ),
+            (
+                ArrowDataType::Timestamp(ArrowTimeUnit::Second, None),
+                DataType::Datetime(TimeUnit::Milliseconds, None),
+            ),
+            (
+                ArrowDataType::Timestamp(ArrowTimeUnit::Second, Some("".to_string())),
+                DataType::Datetime(TimeUnit::Milliseconds, Some("".to_string())),
             ),
             (ArrowDataType::LargeUtf8, DataType::Utf8),
             (ArrowDataType::Utf8, DataType::Utf8),
-            (ArrowDataType::Time64(TimeUnit::Nanosecond), DataType::Time),
-            (ArrowDataType::Time64(TimeUnit::Millisecond), DataType::Time),
-            (ArrowDataType::Time64(TimeUnit::Microsecond), DataType::Time),
-            (ArrowDataType::Time64(TimeUnit::Second), DataType::Time),
-            (ArrowDataType::Time32(TimeUnit::Nanosecond), DataType::Time),
-            (ArrowDataType::Time32(TimeUnit::Millisecond), DataType::Time),
-            (ArrowDataType::Time32(TimeUnit::Microsecond), DataType::Time),
-            (ArrowDataType::Time32(TimeUnit::Second), DataType::Time),
+            (
+                ArrowDataType::Time64(ArrowTimeUnit::Nanosecond),
+                DataType::Time,
+            ),
+            (
+                ArrowDataType::Time64(ArrowTimeUnit::Millisecond),
+                DataType::Time,
+            ),
+            (
+                ArrowDataType::Time64(ArrowTimeUnit::Microsecond),
+                DataType::Time,
+            ),
+            (ArrowDataType::Time64(ArrowTimeUnit::Second), DataType::Time),
+            (
+                ArrowDataType::Time32(ArrowTimeUnit::Nanosecond),
+                DataType::Time,
+            ),
+            (
+                ArrowDataType::Time32(ArrowTimeUnit::Millisecond),
+                DataType::Time,
+            ),
+            (
+                ArrowDataType::Time32(ArrowTimeUnit::Microsecond),
+                DataType::Time,
+            ),
+            (ArrowDataType::Time32(ArrowTimeUnit::Second), DataType::Time),
             (
                 ArrowDataType::List(Box::new(ArrowField::new(
                     "item",
@@ -842,15 +1084,23 @@ mod test {
                 DataType::List(DataType::Float64.into()),
             ),
             (
-                ArrowDataType::Dictionary(IntegerType::UInt32, ArrowDataType::Utf8.into()),
+                ArrowDataType::Dictionary(IntegerType::UInt32, ArrowDataType::Utf8.into(), false),
                 DataType::Categorical,
             ),
             (
-                ArrowDataType::Dictionary(IntegerType::UInt32, ArrowDataType::LargeUtf8.into()),
+                ArrowDataType::Dictionary(
+                    IntegerType::UInt32,
+                    ArrowDataType::LargeUtf8.into(),
+                    false,
+                ),
                 DataType::Categorical,
             ),
             (
-                ArrowDataType::Dictionary(IntegerType::UInt64, ArrowDataType::LargeUtf8.into()),
+                ArrowDataType::Dictionary(
+                    IntegerType::UInt64,
+                    ArrowDataType::LargeUtf8.into(),
+                    false,
+                ),
                 DataType::Categorical,
             ),
         ];
