@@ -12,8 +12,8 @@ use crate::chunked_array::object::extension::create_extension;
 use crate::prelude::*;
 use crate::series::implementations::SeriesWrap;
 use crate::utils::NoNull;
-use arrow::buffer::MutableBuffer;
 use polars_arrow::kernels::take_agg::*;
+use polars_arrow::prelude::QuantileInterpolOptions;
 use polars_arrow::trusted_len::PushUnchecked;
 use std::ops::Deref;
 
@@ -474,11 +474,11 @@ where
         let mut can_fast_explode = true;
         let arr = match self.cont_slice() {
             Ok(values) => {
-                let mut offsets = MutableBuffer::<i64>::with_capacity(groups.len() + 1);
+                let mut offsets = Vec::<i64>::with_capacity(groups.len() + 1);
                 let mut length_so_far = 0i64;
                 offsets.push(length_so_far);
 
-                let mut list_values = MutableBuffer::<T::Native>::with_capacity(self.len());
+                let mut list_values = Vec::<T::Native>::with_capacity(self.len());
                 groups.iter().for_each(|(_, idx)| {
                     let idx_len = idx.len();
                     if idx_len == 0 {
@@ -489,9 +489,8 @@ where
                     // Safety:
                     // group tuples are in bounds
                     unsafe {
-                        list_values.extend_from_trusted_len_iter(
-                            idx.iter().map(|idx| *values.get_unchecked(*idx as usize)),
-                        );
+                        list_values
+                            .extend(idx.iter().map(|idx| *values.get_unchecked(*idx as usize)));
                         // Safety:
                         // we know that offsets has allocated enough slots
                         offsets.push_unchecked(length_so_far);
@@ -558,7 +557,7 @@ impl AggList for Utf8Chunked {
 impl AggList for ListChunked {
     fn agg_list(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
         let mut can_fast_explode = true;
-        let mut offsets = MutableBuffer::<i64>::with_capacity(groups.len() + 1);
+        let mut offsets = Vec::<i64>::with_capacity(groups.len() + 1);
         let mut length_so_far = 0i64;
         offsets.push(length_so_far);
 
@@ -625,34 +624,34 @@ impl AggList for CategoricalChunked {
 impl<T: PolarsObject> AggList for ObjectChunked<T> {
     fn agg_list(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
         let mut can_fast_explode = true;
-        let mut offsets = MutableBuffer::<i64>::with_capacity(groups.len() + 1);
+        let mut offsets = Vec::<i64>::with_capacity(groups.len() + 1);
         let mut length_so_far = 0i64;
         offsets.push(length_so_far);
 
-        let iter = groups
-            .iter()
-            .map(|(_, idx)| {
-                // Safety:
-                // group tuples always in bounds
-                let group_vals =
-                    unsafe { self.take_unchecked((idx.iter().map(|idx| *idx as usize)).into()) };
+        //  we know that iterators length
+        let iter = unsafe {
+            groups
+                .iter()
+                .flat_map(|(_, idx)| {
+                    // Safety:
+                    // group tuples always in bounds
+                    let group_vals =
+                        self.take_unchecked((idx.iter().map(|idx| *idx as usize)).into());
 
-                let idx_len = idx.len();
-                if idx_len == 0 {
-                    can_fast_explode = false;
-                }
-                length_so_far += idx_len as i64;
-                // Safety:
-                // we know that offsets has allocated enough slots
-                unsafe {
+                    let idx_len = idx.len();
+                    if idx_len == 0 {
+                        can_fast_explode = false;
+                    }
+                    length_so_far += idx_len as i64;
+                    // Safety:
+                    // we know that offsets has allocated enough slots
                     offsets.push_unchecked(length_so_far);
-                }
 
-                let arr = group_vals.downcast_iter().next().unwrap().clone();
-                arr.into_iter_cloned()
-            })
-            .flatten()
-            .trust_my_length(self.len());
+                    let arr = group_vals.downcast_iter().next().unwrap().clone();
+                    arr.into_iter_cloned()
+                })
+                .trust_my_length(self.len())
+        };
 
         let mut pe = create_extension(iter);
 
@@ -660,7 +659,7 @@ impl<T: PolarsObject> AggList for ObjectChunked<T> {
         // this is safe because we just created the PolarsExtension
         // meaning that the sentinel is heap allocated and the dereference of the
         // pointer does not fail
-        unsafe { pe.set_to_series_fn::<T>(self.name()) };
+        unsafe { pe.set_to_series_fn::<T>() };
         let extension_array = Arc::new(pe.take_and_forget()) as ArrayRef;
         let extension_dtype = extension_array.data_type();
 
@@ -681,7 +680,12 @@ impl<T: PolarsObject> AggList for ObjectChunked<T> {
 }
 
 pub(crate) trait AggQuantile {
-    fn agg_quantile(&self, _groups: &[(u32, Vec<u32>)], _quantile: f64) -> Option<Series> {
+    fn agg_quantile(
+        &self,
+        _groups: &[(u32, Vec<u32>)],
+        _quantile: f64,
+        _interpol: QuantileInterpolOptions,
+    ) -> Option<Series> {
         None
     }
 
@@ -699,14 +703,19 @@ where
         + arrow::compute::aggregate::SimdOrd<T::Native>,
     ChunkedArray<T>: IntoSeries,
 {
-    fn agg_quantile(&self, groups: &[(u32, Vec<u32>)], quantile: f64) -> Option<Series> {
+    fn agg_quantile(
+        &self,
+        groups: &[(u32, Vec<u32>)],
+        quantile: f64,
+        interpol: QuantileInterpolOptions,
+    ) -> Option<Series> {
         agg_helper::<T, _>(groups, |(_first, idx)| {
             if idx.is_empty() {
                 return None;
             }
 
             let group_vals = unsafe { self.take_unchecked(idx.iter().map(|i| *i as usize).into()) };
-            group_vals.quantile(quantile).unwrap()
+            group_vals.quantile(quantile, interpol).unwrap()
         })
     }
 
