@@ -1,10 +1,12 @@
 use super::GroupTuples;
 use crate::prelude::compare_inner::PartialEqInner;
 use crate::prelude::*;
+use crate::utils::CustomIterTools;
 use crate::vector_hasher::{df_rows_to_hashes_threaded, IdBuildHasher, IdxHash};
 use crate::vector_hasher::{this_partition, AsU64};
 use crate::POOL;
 use crate::{datatypes::PlHashMap, utils::split_df};
+use ahash::CallHasher;
 use hashbrown::hash_map::Entry;
 use hashbrown::{hash_map::RawEntryMut, HashMap};
 use rayon::prelude::*;
@@ -36,7 +38,7 @@ where
         }
     });
 
-    hash_tbl.into_iter().map(|(_k, tpl)| tpl).collect()
+    hash_tbl.into_iter().map(|(_k, tpl)| tpl).collect_trusted()
 }
 
 /// Determine group tuples over different threads. The hash of the key is used to determine the partitions.
@@ -50,7 +52,7 @@ pub(crate) fn groupby_threaded_num<T, IntoSlice>(
     n_partitions: u64,
 ) -> GroupTuples
 where
-    T: Send + Hash + Eq + Sync + Copy + AsU64,
+    T: Send + Hash + Eq + Sync + Copy + AsU64 + CallHasher,
     IntoSlice: AsRef<[T]> + Send + Sync,
 {
     assert!(n_partitions.is_power_of_two());
@@ -69,6 +71,7 @@ where
             for keys in &keys {
                 let keys = keys.as_ref();
                 let len = keys.len() as u32;
+                let hasher = hash_tbl.hasher().clone();
 
                 let mut cnt = 0;
                 keys.iter().for_each(|k| {
@@ -76,15 +79,18 @@ where
                     cnt += 1;
 
                     if this_partition(k.as_u64(), thread_no, n_partitions) {
-                        let entry = hash_tbl.entry(*k);
+                        let hash = T::get_hash(k, &hasher);
+                        let entry = hash_tbl.raw_entry_mut().from_key_hashed_nocheck(hash, k);
 
                         match entry {
-                            Entry::Vacant(entry) => {
+                            RawEntryMut::Vacant(entry) => {
                                 let mut tuples = Vec::with_capacity(group_size_hint);
                                 tuples.push(idx);
-                                entry.insert((idx, tuples));
+                                entry.insert_with_hasher(hash, *k, (idx, tuples), |k| {
+                                    T::get_hash(k, &hasher)
+                                });
                             }
-                            Entry::Occupied(mut entry) => {
+                            RawEntryMut::Occupied(mut entry) => {
                                 let v = entry.get_mut();
                                 v.1.push(idx);
                             }
@@ -93,7 +99,10 @@ where
                 });
                 offset += len;
             }
-            hash_tbl.into_iter().map(|(_k, v)| v).collect::<Vec<_>>()
+            hash_tbl
+                .into_iter()
+                .map(|(_k, v)| v)
+                .collect_trusted::<Vec<_>>()
         })
     })
     .flatten()
