@@ -1,6 +1,5 @@
 use self::hashing::*;
 use crate::chunked_array::builder::PrimitiveChunkedBuilder;
-use crate::frame::select::Selection;
 use crate::prelude::*;
 #[cfg(feature = "groupby_list")]
 use crate::utils::Wrap;
@@ -12,6 +11,7 @@ use crate::POOL;
 use ahash::{CallHasher, RandomState};
 use hashbrown::HashMap;
 use num::NumCast;
+use polars_arrow::prelude::QuantileInterpolOptions;
 use rayon::prelude::*;
 use std::fmt::Debug;
 use std::hash::Hash;
@@ -19,15 +19,25 @@ use std::hash::Hash;
 use std::ops::Deref;
 
 pub mod aggregations;
+#[cfg(feature = "dynamic_groupby")]
+mod dynamic;
 pub(crate) mod hashing;
-#[cfg(feature = "pivot")]
+#[cfg(feature = "rows")]
 pub(crate) mod pivot;
-#[cfg(feature = "downsample")]
-pub mod resample;
+#[cfg(feature = "rows")]
+pub use pivot::PivotAgg;
+#[cfg(not(feature = "dynamic_groupby"))]
+#[derive(Clone, Debug)]
+pub struct DynamicGroupOptions {
+    pub index_column: String,
+}
 
 pub type GroupTuples = Vec<(u32, Vec<u32>)>;
 pub type GroupedMap<T> = HashMap<T, Vec<u32>, RandomState>;
 
+#[cfg(feature = "dynamic_groupby")]
+pub use dynamic::*;
+
 /// Used to create the tuples for a groupby operation.
 pub trait IntoGroupTuples {
     /// Create the tuples need for a groupby operation.
@@ -316,6 +326,7 @@ impl DataFrame {
                 // otherwise we use two bits of this byte to represent null values.
                 let split_0 = split_ca(&$ca0, n_partitions).unwrap();
                 let split_1 = split_ca(&$ca1, n_partitions).unwrap();
+
                 let keys = POOL.install(|| {
                     split_0
                         .into_par_iter()
@@ -324,7 +335,6 @@ impl DataFrame {
                             ca0.into_iter()
                                 .zip(ca1.into_iter())
                                 .map(|(l, r)| $pack_fn(l, r))
-                                .trust_my_length(ca0.len())
                                 .collect_trusted::<Vec<_>>()
                         })
                         .collect::<Vec<_>>()
@@ -422,19 +432,27 @@ impl DataFrame {
     /// ```
     /// use polars_core::prelude::*;
     /// fn groupby_sum(df: &DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("column_name")?
-    ///     .select("agg_column_name")
+    ///     df.groupby(["column_name"])?
+    ///     .select(["agg_column_name"])
     ///     .sum()
     /// }
     /// ```
-    pub fn groupby<'g, J, S: Selection<'g, J>>(&self, by: S) -> Result<GroupBy> {
+    pub fn groupby<I, S>(&self, by: I) -> Result<GroupBy>
+    where
+        I: IntoIterator<Item = S>,
+        S: AsRef<str>,
+    {
         let selected_keys = self.select_series(by)?;
         self.groupby_with_series(selected_keys, true)
     }
 
     /// Group DataFrame using a Series column.
     /// The groups are ordered by their smallest row index.
-    pub fn groupby_stable<'g, J, S: Selection<'g, J>>(&self, by: S) -> Result<GroupBy> {
+    pub fn groupby_stable<I, S>(&self, by: I) -> Result<GroupBy>
+    where
+        I: IntoIterator<Item = S>,
+        S: AsRef<str>,
+    {
         let mut gb = self.groupby(by)?;
         gb.groups.sort_unstable_by_key(|t| t.0);
         Ok(gb)
@@ -462,9 +480,9 @@ impl DataFrame {
 /// let s0 = DateChunked::parse_from_str_slice("date", dates, fmt)
 ///         .into_series();
 /// // create temperature series
-/// let s1 = Series::new("temp", [20, 10, 7, 9, 1].as_ref());
+/// let s1 = Series::new("temp", [20, 10, 7, 9, 1]);
 /// // create rain series
-/// let s2 = Series::new("rain", [0.2, 0.1, 0.3, 0.1, 0.01].as_ref());
+/// let s2 = Series::new("rain", [0.2, 0.1, 0.3, 0.1, 0.01]);
 /// // create a new DataFrame
 /// let df = DataFrame::new(vec![s0, s1, s2]).unwrap();
 /// println!("{:?}", df);
@@ -491,21 +509,21 @@ impl DataFrame {
 /// ```
 ///
 #[derive(Debug, Clone)]
-pub struct GroupBy<'df, 'selection_str> {
+pub struct GroupBy<'df> {
     df: &'df DataFrame,
     pub(crate) selected_keys: Vec<Series>,
     // [first idx, [other idx]]
     pub(crate) groups: GroupTuples,
     // columns selected for aggregation
-    pub(crate) selected_agg: Option<Vec<&'selection_str str>>,
+    pub(crate) selected_agg: Option<Vec<String>>,
 }
 
-impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
+impl<'df> GroupBy<'df> {
     pub fn new(
         df: &'df DataFrame,
         by: Vec<Series>,
         groups: GroupTuples,
-        selected_agg: Option<Vec<&'selection_str str>>,
+        selected_agg: Option<Vec<String>>,
     ) -> Self {
         GroupBy {
             df,
@@ -520,11 +538,14 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     ///
     /// Note that making a selection with this method is not required. If you
     /// skip it all columns (except for the keys) will be selected for aggregation.
-    pub fn select<S, J>(mut self, selection: S) -> Self
-    where
-        S: Selection<'selection_str, J>,
-    {
-        self.selected_agg = Some(selection.to_selection_vec());
+    #[must_use]
+    pub fn select<I: IntoIterator<Item = S>, S: AsRef<str>>(mut self, selection: I) -> Self {
+        self.selected_agg = Some(
+            selection
+                .into_iter()
+                .map(|s| s.as_ref().to_string())
+                .collect(),
+        );
         self
     }
 
@@ -568,6 +589,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
                     .get_column_names()
                     .into_iter()
                     .filter(|a| !by.contains(a))
+                    .map(|s| s.to_string())
                     .collect()
             }
         };
@@ -584,7 +606,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select(&["temp", "rain"]).mean()
+    ///     df.groupby(["date"])?.select(&["temp", "rain"]).mean()
     /// }
     /// ```
     /// Returns:
@@ -623,7 +645,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").sum()
+    ///     df.groupby(["date"])?.select(["temp"]).sum()
     /// }
     /// ```
     /// Returns:
@@ -662,7 +684,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").min()
+    ///     df.groupby(["date"])?.select(["temp"]).min()
     /// }
     /// ```
     /// Returns:
@@ -700,7 +722,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").max()
+    ///     df.groupby(["date"])?.select(["temp"]).max()
     /// }
     /// ```
     /// Returns:
@@ -738,7 +760,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").first()
+    ///     df.groupby(["date"])?.select(["temp"]).first()
     /// }
     /// ```
     /// Returns:
@@ -774,7 +796,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").last()
+    ///     df.groupby(["date"])?.select(["temp"]).last()
     /// }
     /// ```
     /// Returns:
@@ -810,7 +832,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").n_unique()
+    ///     df.groupby(["date"])?.select(["temp"]).n_unique()
     /// }
     /// ```
     /// Returns:
@@ -847,11 +869,13 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     ///
     /// ```rust
     /// # use polars_core::prelude::*;
+    /// # use polars_arrow::prelude::QuantileInterpolOptions;
+    ///
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").quantile(0.2)
+    ///     df.groupby(["date"])?.select(["temp"]).quantile(0.2, QuantileInterpolOptions::default())
     /// }
     /// ```
-    pub fn quantile(&self, quantile: f64) -> Result<DataFrame> {
+    pub fn quantile(&self, quantile: f64, interpol: QuantileInterpolOptions) -> Result<DataFrame> {
         if !(0.0..=1.0).contains(&quantile) {
             return Err(PolarsError::ComputeError(
                 "quantile should be within 0.0 and 1.0".into(),
@@ -859,8 +883,9 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
         }
         let (mut cols, agg_cols) = self.prepare_agg()?;
         for agg_col in agg_cols {
-            let new_name = fmt_groupby_column(agg_col.name(), GroupByMethod::Quantile(quantile));
-            let opt_agg = agg_col.agg_quantile(&self.groups, quantile);
+            let new_name =
+                fmt_groupby_column(agg_col.name(), GroupByMethod::Quantile(quantile, interpol));
+            let opt_agg = agg_col.agg_quantile(&self.groups, quantile, interpol);
             if let Some(mut agg) = opt_agg {
                 agg.rename(&new_name);
                 cols.push(agg.into_series());
@@ -876,7 +901,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").median()
+    ///     df.groupby(["date"])?.select(["temp"]).median()
     /// }
     /// ```
     pub fn median(&self) -> Result<DataFrame> {
@@ -927,7 +952,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.select("temp").count()
+    ///     df.groupby(["date"])?.select(["temp"]).count()
     /// }
     /// ```
     /// Returns:
@@ -967,7 +992,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// ```rust
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
-    ///     df.groupby("date")?.groups()
+    ///     df.groupby(["date"])?.groups()
     /// }
     /// ```
     /// Returns:
@@ -1020,7 +1045,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     ///  ```rust
     ///  # use polars_core::prelude::*;
     ///  fn example(df: DataFrame) -> Result<DataFrame> {
-    ///      df.groupby("date")?.agg(&[("temp", &["n_unique", "sum", "min"])])
+    ///      df.groupby(["date"])?.agg(&[("temp", &["n_unique", "sum", "min"])])
     ///  }
     ///  ```
     ///  Returns:
@@ -1116,7 +1141,7 @@ impl<'df, 'selection_str> GroupBy<'df, 'selection_str> {
     /// # use polars_core::prelude::*;
     /// fn example(df: DataFrame) -> Result<DataFrame> {
     ///     // GroupBy and aggregate to Lists
-    ///     df.groupby("date")?.select("temp").agg_list()
+    ///     df.groupby(["date"])?.select(["temp"]).agg_list()
     /// }
     /// ```
     /// Returns:
@@ -1214,7 +1239,7 @@ pub enum GroupByMethod {
     Sum,
     Groups,
     NUnique,
-    Quantile(f64),
+    Quantile(f64, QuantileInterpolOptions),
     Count,
     List,
     Std,
@@ -1236,7 +1261,7 @@ pub fn fmt_groupby_column(name: &str, method: GroupByMethod) -> String {
         NUnique => format!("{}_n_unique", name),
         Count => format!("{}_count", name),
         List => format!("{}_agg_list", name),
-        Quantile(quantile) => format!("{}_quantile_{:.2}", name, quantile),
+        Quantile(quantile, _interpol) => format!("{}_quantile_{:.2}", name, quantile),
         Std => format!("{}_agg_std", name),
         Var => format!("{}_agg_var", name),
     }
@@ -1244,8 +1269,6 @@ pub fn fmt_groupby_column(name: &str, method: GroupByMethod) -> String {
 
 #[cfg(test)]
 mod test {
-    use itertools::Itertools;
-
     use crate::frame::groupby::{groupby, groupby_threaded_num};
     use crate::prelude::*;
     use crate::utils::split_ca;
@@ -1254,7 +1277,7 @@ mod test {
     #[test]
     #[cfg(feature = "dtype-date")]
     #[cfg_attr(miri, ignore)]
-    fn test_group_by() {
+    fn test_group_by() -> Result<()> {
         let s0 = DateChunked::parse_from_str_slice(
             "date",
             &[
@@ -1267,94 +1290,44 @@ mod test {
             "%Y-%m-%d",
         )
         .into_series();
-        let s1 = Series::new("temp", [20, 10, 7, 9, 1].as_ref());
-        let s2 = Series::new("rain", [0.2, 0.1, 0.3, 0.1, 0.01].as_ref());
+        let s1 = Series::new("temp", [20, 10, 7, 9, 1]);
+        let s2 = Series::new("rain", [0.2, 0.1, 0.3, 0.1, 0.01]);
         let df = DataFrame::new(vec![s0, s1, s2]).unwrap();
-        println!("{:?}", df);
 
-        println!(
-            "{:?}",
-            df.groupby("date").unwrap().select("temp").count().unwrap()
+        let out = df.groupby_stable(["date"])?.select(["temp"]).count()?;
+        assert_eq!(
+            out.column("temp_count")?,
+            &Series::new("temp_count", [2u32, 2, 1])
         );
+
         // Select multiple
-        println!(
-            "{:?}",
-            df.groupby("date")
-                .unwrap()
-                .select(&["temp", "rain"])
-                .mean()
-                .unwrap()
+        let out = df
+            .groupby_stable(["date"])?
+            .select(&["temp", "rain"])
+            .mean()?;
+        assert_eq!(
+            out.column("temp_mean")?,
+            &Series::new("temp_mean", [15.0f64, 4.0, 9.0])
         );
+
         // Group by multiple
-        println!(
-            "multiple keys {:?}",
-            df.groupby(&["date", "temp"])
-                .unwrap()
-                .select("rain")
-                .mean()
-                .unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date").unwrap().select("temp").sum().unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date").unwrap().select("temp").min().unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date").unwrap().select("temp").max().unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date")
-                .unwrap()
-                .select("temp")
-                .agg_list()
-                .unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date").unwrap().select("temp").first().unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date").unwrap().select("temp").last().unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date")
-                .unwrap()
-                .select("temp")
-                .n_unique()
-                .unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date")
-                .unwrap()
-                .select("temp")
-                .quantile(0.2)
-                .unwrap()
-        );
-        println!(
-            "{:?}",
-            df.groupby("date").unwrap().select("temp").median().unwrap()
+        let out = df
+            .groupby_stable(&["date", "temp"])?
+            .select(["rain"])
+            .mean()?;
+        assert!(out.column("rain_mean").is_ok());
+
+        let out = df.groupby_stable(["date"])?.select(["temp"]).sum()?;
+        assert_eq!(
+            out.column("temp_sum")?,
+            &Series::new("temp_sum", [30, 8, 9])
         );
+
         // implicit select all and only aggregate on methods that support that aggregation
-        let gb = df.groupby("date").unwrap().n_unique().unwrap();
-        println!("{:?}", df.groupby("date").unwrap().n_unique().unwrap());
+        let gb = df.groupby(["date"]).unwrap().n_unique().unwrap();
         // check the group by column is filtered out.
         assert_eq!(gb.width(), 3);
-        println!(
-            "{:?}",
-            df.groupby("date")
-                .unwrap()
-                .agg(&[("temp", &["n_unique", "sum", "min"])])
-                .unwrap()
-        );
-        println!("{:?}", df.groupby("date").unwrap().groups().unwrap());
+        Ok(())
     }
 
     #[test]
@@ -1377,19 +1350,16 @@ mod test {
 
         let df =
             DataFrame::new(vec![s0, s1, s2, s3, s4, s5, s6, s7, s8, s9, s10, s11, s12]).unwrap();
-        println!("{:?}", df);
 
         let adf = df
             .groupby(&[
                 "G1", "G2", "G3", "G4", "G5", "G6", "G7", "G8", "G9", "G10", "G11", "G12",
             ])
             .unwrap()
-            .select("N")
+            .select(["N"])
             .sum()
             .unwrap();
 
-        println!("{:?}", adf);
-
         assert_eq!(
             Vec::from(&adf.column("N_sum").unwrap().i32().unwrap().sort(false)),
             &[Some(1), Some(2), Some(2), Some(6)]
@@ -1422,16 +1392,14 @@ mod test {
 
         // Creat the dataframe with the computed series.
         let df = DataFrame::new(series).unwrap();
-        println!("{:?}", df);
 
         // Compute the aggregated DataFrame by the 13 columns defined in `series_names`.
         let adf = df
             .groupby(&series_names)
             .unwrap()
-            .select("N")
+            .select(["N"])
             .sum()
             .unwrap();
-        println!("{:?}", adf);
 
         // Check that the results of the group-by are correct. The content of every column
         // is equal, then, the grouped columns shall be equal and in the same order.
@@ -1456,8 +1424,8 @@ mod test {
                     "val" => [1, 1, 1, 1, 1]
         }
         .unwrap();
-        let res = df.groupby("flt").unwrap().sum().unwrap();
-        let res = res.sort("flt", false).unwrap();
+        let res = df.groupby(["flt"]).unwrap().sum().unwrap();
+        let res = res.sort(["flt"], false).unwrap();
         assert_eq!(
             Vec::from(res.column("val_sum").unwrap().i32().unwrap()),
             &[Some(2), Some(2), Some(1)]
@@ -1479,9 +1447,9 @@ mod test {
 
         // check multiple keys and categorical
         let res = df
-            .groupby_stable(&["foo", "ham"])
+            .groupby_stable(["foo", "ham"])
             .unwrap()
-            .select("bar")
+            .select(["bar"])
             .sum()
             .unwrap();
 
@@ -1500,8 +1468,8 @@ mod test {
         }
         .unwrap();
 
-        let out = df.groupby("a").unwrap().apply(Ok).unwrap();
-        assert!(out.sort("b", false).unwrap().frame_equal(&df));
+        let out = df.groupby(["a"]).unwrap().apply(Ok).unwrap();
+        assert!(out.sort(["b"], false).unwrap().frame_equal(&df));
     }
 
     #[test]
@@ -1512,16 +1480,17 @@ mod test {
             vec![1, 2, 3, 4, 4, 4, 2, 1, 1],
             vec![1, 2, 3, 4, 4, 4],
         ] {
-            let ca = UInt32Chunked::new_from_slice("", slice);
+            let ca = UInt32Chunked::new("", slice);
             let split = split_ca(&ca, 4).unwrap();
 
-            let a = groupby(ca.into_iter()).into_iter().sorted().collect_vec();
+            let mut a = groupby(ca.into_iter()).into_iter().collect::<Vec<_>>();
+            a.sort();
 
             let keys = split.iter().map(|ca| ca.cont_slice().unwrap()).collect();
-            let b = groupby_threaded_num(keys, 0, split.len() as u64)
+            let mut b = groupby_threaded_num(keys, 0, split.len() as u64)
                 .into_iter()
-                .sorted()
-                .collect_vec();
+                .collect::<Vec<_>>();
+            b.sort();
 
             assert_eq!(a, b);
         }
@@ -1534,7 +1503,7 @@ mod test {
             "a" => ["a", "a", "a", "b", "b"],
             "b" => [Some(1), Some(2), None, None, Some(1)]
         )?;
-        let out = df.groupby_stable("a")?.mean()?;
+        let out = df.groupby_stable(["a"])?.mean()?;
 
         assert_eq!(
             Vec::from(out.column("b_mean")?.f64()?),
@@ -1553,17 +1522,17 @@ mod test {
             "int" => [1, 2, 3]
         ]?;
 
-        let out = df.groupby("g")?.select("int").var()?;
+        let out = df.groupby(["g"])?.select(["int"]).var()?;
         assert_eq!(
-            out.column("int_agg_var")?.f64()?.sort(false).get(1),
+            out.column("int_agg_var")?.f64()?.sort(false).get(0),
             Some(0.5)
         );
-        let out = df.groupby("g")?.select("int").std()?;
+        let out = df.groupby(["g"])?.select(["int"]).std()?;
         let val = out
             .column("int_agg_std")?
             .f64()?
             .sort(false)
-            .get(1)
+            .get(0)
             .unwrap();
         let expected = f64::FRAC_1_SQRT_2();
         assert!((val - expected).abs() < 0.000001);
@@ -1581,10 +1550,9 @@ mod test {
             "int" => [1, 2, 3, 1, 1]
         ]?;
 
-        df.may_apply("g", |s| s.cast(&DataType::Categorical))?;
+        df.try_apply("g", |s| s.cast(&DataType::Categorical))?;
 
-        let out = df.groupby("g")?.sum()?;
-        dbg!(out);
+        let _ = df.groupby(["g"])?.sum()?;
         Ok(())
     }
 }
