@@ -11,17 +11,18 @@ pub use crate::{
             PrimitiveChunkedBuilder, Utf8ChunkedBuilder,
         },
         iterator::{IntoNoNullIterator, PolarsIterator},
-        ops::{aggregate::*, chunkops::ChunkOps, *},
+        ops::{aggregate::*, *},
         ChunkedArray,
     },
     datatypes,
     datatypes::*,
     df,
     error::{PolarsError, Result},
-    frame::{hash_join::JoinType, DataFrame},
+    frame::{groupby::GroupTuples, hash_join::JoinType, DataFrame},
+    named_from::NamedFrom,
     series::{
         arithmetic::{LhsNumOps, NumOpsDispatch},
-        IntoSeries, NamedFrom, Series, SeriesTrait,
+        IntoSeries, Series, SeriesTrait,
     },
     testing::*,
     utils::IntoVec,
@@ -29,7 +30,7 @@ pub use crate::{
 };
 pub(crate) use arrow::array::*;
 pub use arrow::datatypes::{Field as ArrowField, Schema as ArrowSchema};
-pub use polars_arrow::prelude::{AlignedVec, LargeListArray, LargeStringArray};
+pub use polars_arrow::prelude::{LargeListArray, LargeStringArray, QuantileInterpolOptions};
 pub(crate) use polars_arrow::trusted_len::TrustedLen;
 pub use std::sync::Arc;
 
@@ -41,7 +42,15 @@ pub use crate::chunked_array::temporal::conversion::*;
 pub use crate::series::arithmetic::checked::NumOpsDispatchChecked;
 
 #[cfg(feature = "rank")]
-pub use crate::chunked_array::ops::unique::rank::RankMethod;
+pub use crate::chunked_array::ops::unique::rank::{RankMethod, RankOptions};
 
 #[cfg(feature = "rolling_window")]
 pub use crate::chunked_array::ops::rolling_window::RollingOptions;
+
+#[cfg(feature = "dynamic_groupby")]
+pub use polars_time::{groupby::ClosedWindow, Duration};
+
+#[cfg(feature = "ewma")]
+pub use polars_arrow::kernels::ewm::EWMOptions;
+
+pub(crate) use polars_arrow::export::*;
