@@ -7,6 +7,10 @@ mod categorical;
     feature = "dtype-time"
 ))]
 mod dates_time;
+#[cfg(feature = "dtype-datetime")]
+mod datetime;
+#[cfg(feature = "dtype-duration")]
+mod duration;
 mod floats;
 mod list;
 #[cfg(feature = "object")]
@@ -31,8 +35,6 @@ use crate::chunked_array::{
     AsSinglePtr, ChunkIdIter,
 };
 use crate::fmt::FmtList;
-#[cfg(feature = "pivot")]
-use crate::frame::groupby::pivot::*;
 use crate::frame::groupby::*;
 use crate::frame::hash_join::{HashJoin, ZipOuterJoinColumn};
 use crate::prelude::*;
@@ -40,6 +42,7 @@ use crate::prelude::*;
 use crate::series::arithmetic::checked::NumOpsDispatchChecked;
 use ahash::RandomState;
 use arrow::array::ArrayRef;
+use polars_arrow::prelude::QuantileInterpolOptions;
 use std::borrow::Cow;
 use std::ops::Deref;
 use std::ops::{BitAnd, BitOr, BitXor};
@@ -86,6 +89,19 @@ macro_rules! impl_dyn_series {
                 self.0.rolling_sum(options)
             }
             #[cfg(feature = "rolling_window")]
+            fn _rolling_median(&self, options: RollingOptions) -> Result<Series> {
+                self.0.rolling_median(options)
+            }
+            #[cfg(feature = "rolling_window")]
+            fn _rolling_quantile(
+                &self,
+                quantile: f64,
+                interpolation: QuantileInterpolOptions,
+                options: RollingOptions,
+            ) -> Result<Series> {
+                self.0.rolling_quantile(quantile, interpolation, options)
+            }
+            #[cfg(feature = "rolling_window")]
             fn _rolling_min(&self, options: RollingOptions) -> Result<Series> {
                 self.0.rolling_min(options)
             }
@@ -120,16 +136,6 @@ macro_rules! impl_dyn_series {
                 self.0.cummin(reverse).into_series()
             }
 
-            #[cfg(feature = "cum_agg")]
-            fn _cumsum(&self, reverse: bool) -> Series {
-                self.0.cumsum(reverse).into_series()
-            }
-
-            #[cfg(feature = "cum_agg")]
-            fn _cumprod(&self, reverse: bool) -> Series {
-                self.0.cumprod(reverse).into_series()
-            }
-
             #[cfg(feature = "asof_join")]
             fn join_asof(&self, other: &Series) -> Result<Vec<Option<u32>>> {
                 self.0.join_asof(other)
@@ -160,7 +166,7 @@ macro_rules! impl_dyn_series {
                 (&self.0).into_partial_ord_inner()
             }
 
-            fn vec_hash(&self, random_state: RandomState) -> AlignedVec<u64> {
+            fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
                 self.0.vec_hash(random_state)
             }
 
@@ -181,7 +187,11 @@ macro_rules! impl_dyn_series {
             }
 
             fn agg_sum(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
-                self.0.agg_sum(groups)
+                use DataType::*;
+                match self.dtype() {
+                    Int8 | UInt8 | Int16 | UInt16 => self.cast(&Int64).unwrap().agg_sum(groups),
+                    _ => self.0.agg_sum(groups),
+                }
             }
 
             fn agg_first(&self, groups: &[(u32, Vec<u32>)]) -> Series {
@@ -208,8 +218,13 @@ macro_rules! impl_dyn_series {
                 self.0.agg_list(groups)
             }
 
-            fn agg_quantile(&self, groups: &[(u32, Vec<u32>)], quantile: f64) -> Option<Series> {
-                self.0.agg_quantile(groups, quantile)
+            fn agg_quantile(
+                &self,
+                groups: &[(u32, Vec<u32>)],
+                quantile: f64,
+                interpol: QuantileInterpolOptions,
+            ) -> Option<Series> {
+                self.0.agg_quantile(groups, quantile, interpol)
             }
 
             fn agg_median(&self, groups: &[(u32, Vec<u32>)]) -> Option<Series> {
@@ -220,26 +235,6 @@ macro_rules! impl_dyn_series {
                 self.0.agg_valid_count(groups)
             }
 
-            #[cfg(feature = "pivot")]
-            fn pivot<'a>(
-                &self,
-                pivot_series: &'a Series,
-                keys: Vec<Series>,
-                groups: &[(u32, Vec<u32>)],
-                agg_type: PivotAgg,
-            ) -> Result<DataFrame> {
-                self.0.pivot(pivot_series, keys, groups, agg_type)
-            }
-
-            #[cfg(feature = "pivot")]
-            fn pivot_count<'a>(
-                &self,
-                pivot_series: &'a Series,
-                keys: Vec<Series>,
-                groups: &[(u32, Vec<u32>)],
-            ) -> Result<DataFrame> {
-                self.0.pivot_count(pivot_series, keys, groups)
-            }
             fn hash_join_inner(&self, other: &Series) -> Vec<(u32, u32)> {
                 HashJoin::hash_join_inner(&self.0, other.as_ref().as_ref())
             }
@@ -279,21 +274,16 @@ macro_rules! impl_dyn_series {
             fn argsort_multiple(&self, by: &[Series], reverse: &[bool]) -> Result<UInt32Chunked> {
                 self.0.argsort_multiple(by, reverse)
             }
-
-            fn str_value(&self, index: usize) -> Cow<str> {
-                // get AnyValue
-                Cow::Owned(format!("{}", self.get(index)))
-            }
         }
 
         impl SeriesTrait for SeriesWrap<$ca> {
             #[cfg(feature = "rolling_window")]
             fn rolling_apply(
                 &self,
-                _window_size: usize,
                 _f: &dyn Fn(&Series) -> Series,
+                _options: RollingOptions,
             ) -> Result<Series> {
-                ChunkRollApply::rolling_apply(&self.0, _window_size, _f).map(|ca| ca.into_series())
+                ChunkRollApply::rolling_apply(&self.0, _f, _options).map(|ca| ca.into_series())
             }
 
             #[cfg(feature = "interpolate")]
@@ -579,14 +569,6 @@ macro_rules! impl_dyn_series {
                 ChunkOps::rechunk(&self.0).into_series()
             }
 
-            fn head(&self, length: Option<usize>) -> Series {
-                self.0.head(length).into_series()
-            }
-
-            fn tail(&self, length: Option<usize>) -> Series {
-                self.0.tail(length).into_series()
-            }
-
             fn expand_at_index(&self, index: usize, length: usize) -> Series {
                 ChunkExpandAtIndex::expand_at_index(&self.0, index, length).into_series()
             }
@@ -612,12 +594,8 @@ macro_rules! impl_dyn_series {
                 self.0.get_any_value_unchecked(index)
             }
 
-            fn sort_in_place(&mut self, reverse: bool) {
-                ChunkSort::sort_in_place(&mut self.0, reverse);
-            }
-
-            fn sort(&self, reverse: bool) -> Series {
-                ChunkSort::sort(&self.0, reverse).into_series()
+            fn sort_with(&self, options: SortOptions) -> Series {
+                ChunkSort::sort_with(&self.0, options).into_series()
             }
 
             fn argsort(&self, reverse: bool) -> UInt32Chunked {
@@ -684,7 +662,7 @@ macro_rules! impl_dyn_series {
                 ChunkFillNull::fill_null(&self.0, strategy).map(|ca| ca.into_series())
             }
 
-            fn sum_as_series(&self) -> Series {
+            fn _sum_as_series(&self) -> Series {
                 ChunkAggSeries::sum_as_series(&self.0)
             }
             fn max_as_series(&self) -> Series {
@@ -705,8 +683,12 @@ macro_rules! impl_dyn_series {
             fn std_as_series(&self) -> Series {
                 VarAggSeries::std_as_series(&self.0)
             }
-            fn quantile_as_series(&self, quantile: f64) -> Result<Series> {
-                ChunkAggSeries::quantile_as_series(&self.0, quantile)
+            fn quantile_as_series(
+                &self,
+                quantile: f64,
+                interpol: QuantileInterpolOptions,
+            ) -> Result<Series> {
+                ChunkAggSeries::quantile_as_series(&self.0, quantile, interpol)
             }
 
             fn fmt_list(&self) -> String {
@@ -716,22 +698,6 @@ macro_rules! impl_dyn_series {
                 Arc::new(SeriesWrap(Clone::clone(&self.0)))
             }
 
-            #[cfg(feature = "random")]
-            #[cfg_attr(docsrs, doc(cfg(feature = "random")))]
-            fn sample_n(&self, n: usize, with_replacement: bool) -> Result<Series> {
-                self.0
-                    .sample_n(n, with_replacement)
-                    .map(|ca| ca.into_series())
-            }
-
-            #[cfg(feature = "random")]
-            #[cfg_attr(docsrs, doc(cfg(feature = "random")))]
-            fn sample_frac(&self, frac: f64, with_replacement: bool) -> Result<Series> {
-                self.0
-                    .sample_frac(frac, with_replacement)
-                    .map(|ca| ca.into_series())
-            }
-
             fn pow(&self, exponent: f64) -> Result<Series> {
                 let f_err = || {
                     Err(PolarsError::InvalidOperation(
