@@ -31,6 +31,15 @@ impl<T> PrivateSeries for SeriesWrap<ObjectChunked<T>>
 where
     T: PolarsObject,
 {
+    fn get_list_builder(
+        &self,
+        _name: &str,
+        _values_capacity: usize,
+        _list_capacity: usize,
+    ) -> Box<dyn ListBuilderTrait> {
+        ObjectChunked::<T>::get_list_builder(_name, _values_capacity, _list_capacity)
+    }
+
     fn _field(&self) -> Cow<Field> {
         Cow::Borrowed(self.0.ref_field())
     }
@@ -46,17 +55,11 @@ where
         self.0.agg_list(groups)
     }
 
-    fn str_value(&self, index: usize) -> Cow<str> {
-        match (&self.0).get(index) {
-            None => Cow::Borrowed("null"),
-            Some(val) => Cow::Owned(format!("{}", val)),
-        }
-    }
     fn into_partial_eq_inner<'a>(&'a self) -> Box<dyn PartialEqInner + 'a> {
         (&self.0).into_partial_eq_inner()
     }
 
-    fn vec_hash(&self, random_state: RandomState) -> AlignedVec<u64> {
+    fn vec_hash(&self, random_state: RandomState) -> Vec<u64> {
         self.0.vec_hash(random_state)
     }
 
@@ -156,14 +159,6 @@ where
         ChunkOps::rechunk(&self.0).into_series()
     }
 
-    fn head(&self, length: Option<usize>) -> Series {
-        ObjectChunked::head(&self.0, length).into_series()
-    }
-
-    fn tail(&self, length: Option<usize>) -> Series {
-        ObjectChunked::tail(&self.0, length).into_series()
-    }
-
     fn take_every(&self, n: usize) -> Series {
         self.0.take_every(n).into_series()
     }
@@ -241,18 +236,6 @@ where
         Arc::new(SeriesWrap(Clone::clone(&self.0)))
     }
 
-    #[cfg(feature = "random")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "random")))]
-    fn sample_n(&self, n: usize, with_replacement: bool) -> Result<Series> {
-        ObjectChunked::sample_n(&self.0, n, with_replacement).map(|ca| ca.into_series())
-    }
-
-    #[cfg(feature = "random")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "random")))]
-    fn sample_frac(&self, frac: f64, with_replacement: bool) -> Result<Series> {
-        ObjectChunked::sample_frac(&self.0, frac, with_replacement).map(|ca| ca.into_series())
-    }
-
     fn get_object(&self, index: usize) -> Option<&dyn PolarsObjectSafe> {
         ObjectChunked::<T>::get_object(&self.0, index)
     }
@@ -261,7 +244,7 @@ where
         &self.0
     }
 
-    fn sum_as_series(&self) -> Series {
+    fn _sum_as_series(&self) -> Series {
         ObjectChunked::<T>::full_null(self.name(), 1).into_series()
     }
     fn max_as_series(&self) -> Series {
