@@ -1,10 +1,10 @@
-pub use crate::prelude::ChunkCompare;
-use crate::prelude::*;
-use arrow::array::ArrayRef;
-
 #[cfg(feature = "object")]
 use crate::chunked_array::object::PolarsObjectSafe;
 use crate::chunked_array::ChunkIdIter;
+pub use crate::prelude::ChunkCompare;
+use crate::prelude::*;
+use arrow::array::ArrayRef;
+use polars_arrow::prelude::QuantileInterpolOptions;
 #[cfg(feature = "object")]
 use std::any::Any;
 use std::borrow::Cow;
@@ -34,8 +34,7 @@ macro_rules! invalid_operation_panic {
 
 pub(crate) mod private {
     use super::*;
-    #[cfg(feature = "pivot")]
-    use crate::frame::groupby::pivot::PivotAgg;
+    #[cfg(feature = "rows")]
     use crate::frame::groupby::GroupTuples;
 
     use crate::chunked_array::ops::compare_inner::{PartialEqInner, PartialOrdInner};
@@ -43,7 +42,7 @@ pub(crate) mod private {
 
     pub trait PrivateSeriesNumeric {
         fn bit_repr_is_large(&self) -> bool {
-            unimplemented!()
+            false
         }
         fn bit_repr_large(&self) -> UInt64Chunked {
             unimplemented!()
@@ -54,6 +53,16 @@ pub(crate) mod private {
     }
 
     pub trait PrivateSeries {
+        #[cfg(feature = "object")]
+        fn get_list_builder(
+            &self,
+            _name: &str,
+            _values_capacity: usize,
+            _list_capacity: usize,
+        ) -> Box<dyn ListBuilderTrait> {
+            invalid_operation_panic!(self)
+        }
+
         /// Get field (used in schema)
         fn _field(&self) -> Cow<Field> {
             invalid_operation_panic!(self)
@@ -73,29 +82,45 @@ pub(crate) mod private {
         fn _rolling_mean(&self, _options: RollingOptions) -> Result<Series> {
             invalid_operation!(self)
         }
-        /// Apply a rolling sum to a Series. See:
+        /// Apply a rolling sum to a Series.
         #[cfg(feature = "rolling_window")]
         fn _rolling_sum(&self, _options: RollingOptions) -> Result<Series> {
             invalid_operation!(self)
         }
-        /// Apply a rolling min to a Series. See:
+        /// Apply a rolling median to a Series.
+        #[cfg(feature = "rolling_window")]
+        fn _rolling_median(&self, _options: RollingOptions) -> Result<Series> {
+            invalid_operation!(self)
+        }
+        /// Apply a rolling quantile to a Series.
+        #[cfg(feature = "rolling_window")]
+        fn _rolling_quantile(
+            &self,
+            _quantile: f64,
+            _interpolation: QuantileInterpolOptions,
+            _options: RollingOptions,
+        ) -> Result<Series> {
+            invalid_operation!(self)
+        }
+
+        /// Apply a rolling min to a Series.
         #[cfg(feature = "rolling_window")]
         fn _rolling_min(&self, _options: RollingOptions) -> Result<Series> {
             invalid_operation!(self)
         }
-        /// Apply a rolling max to a Series. See:
+        /// Apply a rolling max to a Series.
         #[cfg(feature = "rolling_window")]
         fn _rolling_max(&self, _options: RollingOptions) -> Result<Series> {
             invalid_operation!(self)
         }
 
-        /// Apply a rolling variance to a Series. See:
+        /// Apply a rolling variance to a Series.
         #[cfg(feature = "rolling_window")]
         fn _rolling_var(&self, _options: RollingOptions) -> Result<Series> {
             invalid_operation!(self)
         }
 
-        /// Apply a rolling std_dev to a Series. See:
+        /// Apply a rolling std_dev to a Series.
         #[cfg(feature = "rolling_window")]
         fn _rolling_std(&self, _options: RollingOptions) -> Result<Series> {
             invalid_operation!(self)
@@ -113,18 +138,6 @@ pub(crate) mod private {
             panic!("operation cummin not supported for this dtype")
         }
 
-        /// Get an array with the cumulative sum computed at every element
-        #[cfg(feature = "cum_agg")]
-        fn _cumsum(&self, _reverse: bool) -> Series {
-            panic!("operation cumsum not supported for this dtype")
-        }
-
-        /// Get an array with the cumulative sum computed at every element
-        #[cfg(feature = "cum_agg")]
-        fn _cumprod(&self, _reverse: bool) -> Series {
-            panic!("operation cumprod not supported for this dtype")
-        }
-
         #[cfg(feature = "asof_join")]
         fn join_asof(&self, _other: &Series) -> Result<Vec<Option<u32>>> {
             invalid_operation!(self)
@@ -150,7 +163,7 @@ pub(crate) mod private {
         fn into_partial_ord_inner<'a>(&'a self) -> Box<dyn PartialOrdInner + 'a> {
             invalid_operation_panic!(self)
         }
-        fn vec_hash(&self, _build_hasher: RandomState) -> AlignedVec<u64> {
+        fn vec_hash(&self, _build_hasher: RandomState) -> Vec<u64> {
             invalid_operation_panic!(self)
         }
         fn vec_hash_combine(&self, _build_hasher: RandomState, _hashes: &mut [u64]) {
@@ -165,6 +178,8 @@ pub(crate) mod private {
         fn agg_max(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
             None
         }
+        /// If the [`DataType`] is one of `{Int8, UInt8, Int16, UInt16}` the `Series` is
+        /// first cast to `Int64` to prevent overflow issues.
         fn agg_sum(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
             None
         }
@@ -186,7 +201,12 @@ pub(crate) mod private {
         fn agg_list(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
             None
         }
-        fn agg_quantile(&self, _groups: &[(u32, Vec<u32>)], _quantile: f64) -> Option<Series> {
+        fn agg_quantile(
+            &self,
+            _groups: &[(u32, Vec<u32>)],
+            _quantile: f64,
+            _interpol: QuantileInterpolOptions,
+        ) -> Option<Series> {
             None
         }
         fn agg_median(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
@@ -196,27 +216,6 @@ pub(crate) mod private {
         fn agg_valid_count(&self, _groups: &[(u32, Vec<u32>)]) -> Option<Series> {
             None
         }
-        #[cfg(feature = "pivot")]
-        fn pivot<'a>(
-            &self,
-            _pivot_series: &'a Series,
-            _keys: Vec<Series>,
-            _groups: &[(u32, Vec<u32>)],
-            _agg_type: PivotAgg,
-        ) -> Result<DataFrame> {
-            invalid_operation_panic!(self)
-        }
-
-        #[cfg(feature = "pivot")]
-        fn pivot_count<'a>(
-            &self,
-            _pivot_series: &'a Series,
-            _keys: Vec<Series>,
-            _groups: &[(u32, Vec<u32>)],
-        ) -> Result<DataFrame> {
-            invalid_operation_panic!(self)
-        }
-
         fn hash_join_inner(&self, _other: &Series) -> Vec<(u32, u32)> {
             invalid_operation_panic!(self)
         }
@@ -261,10 +260,6 @@ pub(crate) mod private {
                 "argsort_multiple is not implemented for this Series".into(),
             ))
         }
-        /// Formatted string representation. Can used in formatting.
-        fn str_value(&self, _index: usize) -> Cow<str> {
-            invalid_operation_panic!(self)
-        }
     }
 }
 
@@ -451,6 +446,13 @@ pub trait SeriesTrait:
         ))
     }
 
+    /// Unpack to ChunkedArray of dtype duration
+    fn duration(&self) -> Result<&DurationChunked> {
+        Err(PolarsError::SchemaMisMatch(
+            format!("Series dtype {:?} != duration", self.dtype()).into(),
+        ))
+    }
+
     /// Unpack to ChunkedArray of dtype list
     fn list(&self) -> Result<&ListChunked> {
         Err(PolarsError::SchemaMisMatch(
@@ -465,38 +467,6 @@ pub trait SeriesTrait:
         ))
     }
 
-    /// Check if underlying physical data is numeric.
-    ///
-    /// Date types and Categoricals are also considered numeric.
-    fn is_numeric_physical(&self) -> bool {
-        // allow because it cannot be replaced when object feature is activated
-        #[allow(clippy::match_like_matches_macro)]
-        match self.dtype() {
-            DataType::Utf8 | DataType::List(_) | DataType::Boolean | DataType::Null => false,
-            #[cfg(feature = "object")]
-            DataType::Object(_) => false,
-            _ => true,
-        }
-    }
-
-    /// Check if underlying data is numeric
-    fn is_numeric(&self) -> bool {
-        // allow because it cannot be replaced when object feature is activated
-        #[allow(clippy::match_like_matches_macro)]
-        match self.dtype() {
-            DataType::Utf8
-            | DataType::List(_)
-            | DataType::Categorical
-            | DataType::Date
-            | DataType::Datetime
-            | DataType::Boolean
-            | DataType::Null => false,
-            #[cfg(feature = "object")]
-            DataType::Object(_) => false,
-            _ => true,
-        }
-    }
-
     /// Append Arrow array of same dtype to this Series.
     fn append_array(&mut self, _other: ArrayRef) -> Result<()> {
         invalid_operation_panic!(self)
@@ -534,7 +504,8 @@ pub trait SeriesTrait:
     ///
     /// # Safety
     ///
-    /// This doesn't check any bounds.
+    /// - This doesn't check any bounds.
+    /// - Iterator must be TrustedLen
     unsafe fn take_iter_unchecked(&self, _iter: &mut dyn TakeIterator) -> Series {
         invalid_operation_panic!(self)
     }
@@ -551,7 +522,8 @@ pub trait SeriesTrait:
     ///
     /// # Safety
     ///
-    /// This doesn't check any bounds.
+    /// - This doesn't check any bounds.
+    /// - Iterator must be TrustedLen
     unsafe fn take_opt_iter_unchecked(&self, _iter: &mut dyn TakeIteratorNulls) -> Series {
         invalid_operation_panic!(self)
     }
@@ -583,16 +555,6 @@ pub trait SeriesTrait:
         invalid_operation_panic!(self)
     }
 
-    /// Get the head of the Series.
-    fn head(&self, _length: Option<usize>) -> Series {
-        invalid_operation_panic!(self)
-    }
-
-    /// Get the tail of the Series.
-    fn tail(&self, _length: Option<usize>) -> Series {
-        invalid_operation_panic!(self)
-    }
-
     /// Take every nth value as a new Series
     fn take_every(&self, n: usize) -> Series;
 
@@ -659,12 +621,7 @@ pub trait SeriesTrait:
         invalid_operation_panic!(self)
     }
 
-    /// Sort in place.
-    fn sort_in_place(&mut self, _reverse: bool) {
-        invalid_operation_panic!(self)
-    }
-
-    fn sort(&self, _reverse: bool) -> Series {
+    fn sort_with(&self, _options: SortOptions) -> Series {
         invalid_operation_panic!(self)
     }
 
@@ -818,7 +775,10 @@ pub trait SeriesTrait:
     }
 
     /// Get the sum of the Series as a new Series of length 1.
-    fn sum_as_series(&self) -> Series {
+    ///
+    /// If the [`DataType`] is one of `{Int8, UInt8, Int16, UInt16}` the `Series` is
+    /// first cast to `Int64` to prevent overflow issues.
+    fn _sum_as_series(&self) -> Series {
         invalid_operation_panic!(self)
     }
     /// Get the max of the Series as a new Series of length 1.
@@ -846,7 +806,11 @@ pub trait SeriesTrait:
         invalid_operation_panic!(self)
     }
     /// Get the quantile of the ChunkedArray as a new Series of length 1.
-    fn quantile_as_series(&self, _quantile: f64) -> Result<Series> {
+    fn quantile_as_series(
+        &self,
+        _quantile: f64,
+        _interpol: QuantileInterpolOptions,
+    ) -> Result<Series> {
         invalid_operation_panic!(self)
     }
 
@@ -861,7 +825,7 @@ pub trait SeriesTrait:
     fn hour(&self) -> Result<UInt32Chunked> {
         match self.dtype() {
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.hour()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.hour()),
             #[cfg(feature = "dtype-time")]
             DataType::Time => self.time().map(|ca| ca.hour()),
             _ => Err(PolarsError::InvalidOperation(
@@ -877,7 +841,7 @@ pub trait SeriesTrait:
     fn minute(&self) -> Result<UInt32Chunked> {
         match self.dtype() {
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.minute()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.minute()),
             #[cfg(feature = "dtype-time")]
             DataType::Time => self.time().map(|ca| ca.minute()),
             _ => Err(PolarsError::InvalidOperation(
@@ -893,7 +857,7 @@ pub trait SeriesTrait:
     fn second(&self) -> Result<UInt32Chunked> {
         match self.dtype() {
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.second()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.second()),
             #[cfg(feature = "dtype-time")]
             DataType::Time => self.time().map(|ca| ca.second()),
             _ => Err(PolarsError::InvalidOperation(
@@ -909,7 +873,7 @@ pub trait SeriesTrait:
     fn nanosecond(&self) -> Result<UInt32Chunked> {
         match self.dtype() {
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.nanosecond()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.nanosecond()),
             #[cfg(feature = "dtype-time")]
             DataType::Time => self.time().map(|ca| ca.nanosecond()),
             _ => Err(PolarsError::InvalidOperation(
@@ -929,7 +893,7 @@ pub trait SeriesTrait:
             #[cfg(feature = "dtype-date")]
             DataType::Date => self.date().map(|ca| ca.day()),
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.day()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.day()),
             _ => Err(PolarsError::InvalidOperation(
                 format!("operation not supported on dtype {:?}", self.dtype()).into(),
             )),
@@ -943,7 +907,7 @@ pub trait SeriesTrait:
             #[cfg(feature = "dtype-date")]
             DataType::Date => self.date().map(|ca| ca.weekday()),
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.weekday()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.weekday()),
             _ => Err(PolarsError::InvalidOperation(
                 format!("operation not supported on dtype {:?}", self.dtype()).into(),
             )),
@@ -959,7 +923,7 @@ pub trait SeriesTrait:
             #[cfg(feature = "dtype-date")]
             DataType::Date => self.date().map(|ca| ca.week()),
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.week()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.week()),
             _ => Err(PolarsError::InvalidOperation(
                 format!("operation not supported on dtype {:?}", self.dtype()).into(),
             )),
@@ -976,7 +940,7 @@ pub trait SeriesTrait:
             #[cfg(feature = "dtype-date")]
             DataType::Date => self.date().map(|ca| ca.ordinal()),
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.ordinal()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.ordinal()),
             _ => Err(PolarsError::InvalidOperation(
                 format!("operation not supported on dtype {:?}", self.dtype()).into(),
             )),
@@ -994,7 +958,7 @@ pub trait SeriesTrait:
             #[cfg(feature = "dtype-date")]
             DataType::Date => self.date().map(|ca| ca.month()),
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.month()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.month()),
             _ => Err(PolarsError::InvalidOperation(
                 format!("operation not supported on dtype {:?}", self.dtype()).into(),
             )),
@@ -1010,7 +974,7 @@ pub trait SeriesTrait:
             #[cfg(feature = "dtype-date")]
             DataType::Date => self.date().map(|ca| ca.year()),
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.year()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.year()),
             _ => Err(PolarsError::InvalidOperation(
                 format!("operation not supported on dtype {:?}", self.dtype()).into(),
             )),
@@ -1025,7 +989,7 @@ pub trait SeriesTrait:
             #[cfg(feature = "dtype-date")]
             DataType::Date => self.date().map(|ca| ca.strftime(fmt).into_series()),
             #[cfg(feature = "dtype-datetime")]
-            DataType::Datetime => self.datetime().map(|ca| ca.strftime(fmt).into_series()),
+            DataType::Datetime(_, _) => self.datetime().map(|ca| ca.strftime(fmt).into_series()),
             #[cfg(feature = "dtype-time")]
             DataType::Time => self.time().map(|ca| ca.strftime(fmt).into_series()),
             _ => Err(PolarsError::InvalidOperation(
@@ -1040,11 +1004,18 @@ pub trait SeriesTrait:
     fn timestamp(&self) -> Result<Int64Chunked> {
         match self.dtype() {
             DataType::Date => self
-                .cast(&DataType::Int64)
+                .cast(&DataType::Datetime(TimeUnit::Milliseconds, None))
                 .unwrap()
                 .datetime()
-                .map(|ca| (ca.deref() * 1000)),
-            DataType::Datetime => self.datetime().map(|ca| ca.deref().clone()),
+                .map(|ca| (ca.deref().clone())),
+            DataType::Datetime(tu, tz) => {
+                use TimeUnit::*;
+                match (tu, tz) {
+                    (Nanoseconds, None) => self.datetime().map(|ca| ca.deref().clone() / 1_000_000),
+                    (Milliseconds, None) => self.datetime().map(|ca| ca.deref().clone()),
+                    _ => todo!(),
+                }
+            }
             _ => Err(PolarsError::InvalidOperation(
                 format!("operation not supported on dtype {:?}", self.dtype()).into(),
             )),
@@ -1056,16 +1027,6 @@ pub trait SeriesTrait:
         invalid_operation_panic!(self)
     }
 
-    #[cfg(feature = "random")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "random")))]
-    /// Sample n datapoints from this Series.
-    fn sample_n(&self, n: usize, with_replacement: bool) -> Result<Series>;
-
-    #[cfg(feature = "random")]
-    #[cfg_attr(docsrs, doc(cfg(feature = "random")))]
-    /// Sample a fraction between 0.0-1.0 of this Series.
-    fn sample_frac(&self, frac: f64, with_replacement: bool) -> Result<Series>;
-
     #[cfg(feature = "object")]
     #[cfg_attr(docsrs, doc(cfg(feature = "object")))]
     /// Get the value at this index as a downcastable Any trait ref.
@@ -1133,7 +1094,11 @@ pub trait SeriesTrait:
     #[cfg_attr(docsrs, doc(cfg(feature = "rolling_window")))]
     /// Apply a custom function over a rolling/ moving window of the array.
     /// This has quite some dynamic dispatch, so prefer rolling_min, max, mean, sum over this.
-    fn rolling_apply(&self, _window_size: usize, _f: &dyn Fn(&Series) -> Series) -> Result<Series> {
+    fn rolling_apply(
+        &self,
+        _f: &dyn Fn(&Series) -> Series,
+        _options: RollingOptions,
+    ) -> Result<Series> {
         panic!("rolling apply not implemented for this dtype. Only implemented for numeric data.")
     }
     #[cfg(feature = "concat_str")]
